unit Arrow;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, IniFiles;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, IniFiles;
{$endif}

type
  TArrowProperties = class(TForm)
    Panel2: TPanel;
    OK: TButton;
    CancelButton: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    cbStyle: TComboBox;
    Label11: TLabel;
    Label12: TLabel;
    edWidth: TEdit;
    Label5: TLabel;
    edHeadAngle: TEdit;
    Label6: TLabel;
    edHeadLength: TEdit;
    chkPaintHead: TCheckBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    chkUseMouse: TCheckBox;
    chkBothHead: TCheckBox;
    HelpButton: TButton;
    edXstart: TEdit;
    edYstart: TEdit;
    edXdestn: TEdit;
    edYdestn: TEdit;
    procedure OKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chkUseMouseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
  private
  public
  end;

var
  ArrowProperties: TArrowProperties;

implementation

uses MainForm;

{$R *.xfm}

procedure TArrowProperties.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
  procedure GetComboBoxItems(CB: TComboBox );
  begin
    i := CB.ItemIndex;
    CB.Items.Text := StringReplace(IniFile.ReadString(S, CB.Name +  '.Items.Text', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
    CB.ItemIndex := i;
  end;
begin
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption                  := ReadString(S, 'ArrowProperties.Caption', '');
        Label1.Caption           := ReadString(S, 'Label1.Caption', '');
        Label2.Caption           := ReadString(S, 'Label2.Caption', '');
        Label3.Caption           := ReadString(S, 'Label3.Caption', '');
        Label4.Caption           := ReadString(S, 'Label4.Caption', '');
        Label11.Caption          := ReadString(S, 'Label11.Caption', '');
        Label12.Caption          := ReadString(S, 'Label12.Caption', '');
        Label5.Caption           := ReadString(S, 'Label5.Caption', '');
        Label6.Caption           := ReadString(S, 'Label6.Caption', '');
        Label7.Caption           := ReadString(S, 'Label7.Caption', '');
        Label8.Caption           := ReadString(S, 'Label8.Caption', '');
        Label9.Caption           := ReadString(S, 'Label9.Caption', '');
        Label10.Caption          := ReadString(S, 'Label10.Caption', '');
        Panel2.Caption           := ReadString(S, 'Panel2.Caption', '');
        CancelButton.Caption     := ReadString(S, 'CancelButton.Caption', '');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption', '');
        chkPaintHead.Caption     := ReadString(S, 'chkPaintHead.Caption', '');
        chkUseMouse.Caption      := ReadString(S, 'chkUseMouse.Caption', '');
        chkBothHead.Caption      := ReadString(S, 'chkBothHead.Caption', '');
        GetComboBoxItems(cbStyle);
      finally
        Free;
      end;
  end;
end;

procedure TArrowProperties.FormShow(Sender: TObject);
begin
  Top := 0;
  Left := Screen.Width div 2 - Width div 2;
  if Main.tbNewArrow.Down = True then
  begin
    ALstyle[NA] :=  1;
    AHangle[NA] := 12;
    ALwidth[NA] := 0.2;
    AHlength[NA]:= 3;
    AHpaint[NA] := 1;
    ArrowStart[NA,1] := 0;
    ArrowStart[NA,2] := 0;
    ArrowDestination[NA,1] := 0;
    ArrowDestination[NA,2] := 0;
    chkUseMouse.Enabled := True;
    chkUseMouse.Checked := True;
  end
  else
  begin
    chkUseMouse.Checked := False;
    chkUseMouse.Enabled := False;
  end;
  if AHpaint[NA] = 1 then chkPaintHead.Checked := True;
  edXstart.Text := FormatFloat('0.###', ArrowStart[NA,1]);
  edYstart.Text := FormatFloat('0.###', ArrowStart[NA,2]);
  edXdestn.Text := FormatFloat('0.###', ArrowDestination[NA,1]);
  edYdestn.Text := FormatFloat('0.###', ArrowDestination[NA,2]);
  cbStyle.ItemIndex := ALstyle[NA] - 1;
  edHeadAngle.Text := IntToStr(AHangle[NA]);
  edHeadLength.Text := FloatToStr(AHlength[NA]);
  edWidth.Text := FormatFloat('0.0', ALwidth[NA]);
  chkBothHead.Checked := DrawBothHead[NA];
  cbStyle.SetFocus;
end;

procedure TArrowProperties.OKClick(Sender: TObject);
begin
  if chkPaintHead.Checked = True then
    AHpaint[NA] := 1
  else
    AHpaint[NA] := 0;
  ArrowStart[NA,1] := StrToFloat(edXstart.Text);
  ArrowStart[NA,2] := StrToFloat(edYstart.Text);
  ArrowDestination[NA,1] := StrToFloat(edXdestn.Text);
  ArrowDestination[NA,2] := StrToFloat(edYdestn.Text);
  ALstyle[NA] := cbStyle.ItemIndex + 1;
  AHangle[NA] := StrToInt(edHeadAngle.Text);
  AHlength[NA]:= StrToFloat(edHeadLength.Text);
  ALwidth[NA] := StrToFloat(edWidth.Text);
  DrawBothHead[NA] := chkBothHead.Checked;
end;

procedure TArrowProperties.chkUseMouseClick(Sender: TObject);
var b: Boolean;
begin
  b := not chkUseMouse.Checked;
  edXstart.Enabled := b;
  edYstart.Enabled := b;
  edXdestn.Enabled := b;
  edYdestn.Enabled := b;
end;

procedure TArrowProperties.HelpButtonClick(Sender: TObject);
begin
  {$ifdef Linux}
  InvokeHelp;
 // Main.IndexClick(Sender);
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
end;

end.
