unit Axis_Dialog;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QComCtrls, Math, QMenus, 
  Title_Editor, QTypes, Qt, IniFiles, QCalendar
  ;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Math, ToolWin, Menus, Title_Editor, Spin, IniFiles;
{$endif}

type
  TAxisDialog = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Edit4: TEdit;
    Edit5: TEdit;
    Label8: TLabel;
    Edit2: TEdit;
    Label9: TLabel;
    Edit3: TEdit;
    GroupBox1: TGroupBox;
    Label5: TLabel;
    Edit8: TEdit;
    Label6: TLabel;
    Edit9: TEdit;
    Label7: TLabel;
    CheckBox2: TCheckBox;
    CheckBox1: TCheckBox;
    Timer1: TTimer;
    Panel1: TPanel;
    OK: TButton;
    CancelButton: TButton;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CheckBox5: TCheckBox;
    TitleEditor: TTitleEditor;
    CheckBox6: TCheckBox;
    Button1: TButton;
    Panel3: TPanel;
    Image1: TImage;
    ColorDialog1: TColorDialog;
    HelpButton: TButton;
    SpinEdit1: TSpinEdit;
    SpinEdit2: TSpinEdit;
    DateTimePicker: TLTSDatePicker;//TDateTimePicker;
    chkExponential: TCheckBox;
    lblDateFormat: TLabel;
    edDateFormat: TEdit;
    procedure FormShow(Sender: TObject);
    procedure Edit4Change(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure OKClick(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TitleEditorEnter(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Panel3Exit(Sender: TObject);
    procedure Panel3Enter(Sender: TObject);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CheckBox6Click(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure TitleEditorMemoExit(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpinEdit1Changed(Sender: TObject; NewValue: Integer);
    procedure SpinEdit1Change(Sender: TObject);
  private
    procedure FindMaxMin;
  public
    procedure SetDateSeparator(S: String);
  end;

var
  AxisDialog: TAxisDialog;
  TextStore: String;
  Msg: array[1..30] of String;
  ChangeAllowed: Boolean;

implementation

uses MainForm, Legend_Dialog;

{$R *.xfm}

procedure TAxisDialog.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
begin
  Msg[1] := 'Xb axis';
  Msg[2] := 'Xt axis';
  Msg[3] := 'Yl axis';
  Msg[4] := 'Yr axis';
  Msg[5] := 'From';
  Msg[6] := 'to';
  Msg[7] := 'From';
  Msg[8] := 'to';
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Label1.Caption           := ReadString(S, 'Label1.Caption', '');
        Label2.Caption           := ReadString(S, 'Label2.Caption', '');
        Label3.Caption           := ReadString(S, 'Label3.Caption', '');
        Label8.Caption           := ReadString(S, 'Label8.Caption', '');
        Label9.Caption           := ReadString(S, 'Label9.Caption', '');
        Label7.Caption           := ReadString(S, 'Label7.Caption', '');
        GroupBox1.Caption        := ReadString(S, 'GroupBox1.Caption', '');
        Label5.Caption           := ReadString(S, 'Label5.Caption', '');
        Label6.Caption           := ReadString(S, 'Label6.Caption', '');
        lblDateFormat.Caption    := ReadString(S, 'lblDateFormat.Caption', '');
        CheckBox5.Caption        := ReadString(S, 'CheckBox5.Caption', '');
        CheckBox2.Caption        := ReadString(S, 'CheckBox2.Caption', '');
        CheckBox1.Caption        := ReadString(S, 'CheckBox1.Caption', '');
        Panel1.Caption           := ReadString(S, 'Panel1.Caption', '');
        CancelButton.Caption     := ReadString(S, 'CancelButton.Caption', '');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption', '');
        CheckBox3.Caption        := ReadString(S, 'CheckBox3.Caption', '');
        CheckBox4.Caption        := ReadString(S, 'CheckBox4.Caption', '');
        CheckBox6.Caption        := ReadString(S, 'CheckBox6.Caption', '');
        Button1.Caption          := ReadString(S, 'Button1.Caption', '');
        chkExponential.Caption   := ReadString(S, 'chkExponential.Caption', '');
    //    Font.Name                := ReadString(S, 'AxisDialog.Font.Name', '');
        for i := 1 to 10 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  if LANG = 128 then
  begin
    TitleEditor.FontComboBox.Items.Add(Msg[9]);
    TitleEditor.FontComboBox.Items.Add(Msg[10]);
    Label8.Left := Label8.Left + 20;
    Label9.Left := Label9.Left + 20;
    Edit2.Left := Edit2.Left - 30;
    Edit3.Left := Edit3.Left - 30;
    Label5.Left := Label5.Left + 20;
    Label6.Left := Label6.Left + 20;
    Edit8.Left := Edit8.Left - 30;
    Edit9.Left := Edit9.Left - 30;
    Edit4.Left := Edit4.Left + 20;
    Label2.Left := Label2.Left + 10;
    Edit5.Left := Edit5.Left + 43;
    SpinEdit1.Left := SpinEdit1.Left + 30;
    SpinEdit2.Left := SpinEdit2.Left + 55;
    Label7.Left := SpinEdit1.Left + SpinEdit1.Width + 10;
    CheckBox2.Left := CheckBox2.Left + 40;
    CheckBox1.Left := CheckBox2.Left;
    CheckBox3.Left := CheckBox3.Left + 25;
    CheckBox4.Left := CheckBox4.Left + 30;
    DateTimePicker.Left := DateTimePicker.Left - 85;
   // ShortDateFormat := 'mm.dd';
  //  ShowMessage(Font.Name);
  end
  else
  begin
  //  ShortDateFormat := 'dd.mm';
  end;
  //DateSeparator := '.';
end;

procedure TAxisDialog.FormShow(Sender: TObject);
var i : Integer;
begin
  ChangeAllowed := False;
  i := AxisMode;
  Case i of
  1:Caption := Msg[1];
  2:Caption := Msg[2];
  3:Caption := Msg[3];
  4:Caption := Msg[4];
  end;
  if AxisMode < 3 then
  begin
    Label1.Caption := Msg[5];
    Label2.Caption := Msg[6];
  end
  else
  begin
    Label1.Caption := Msg[7];
    Label2.Caption := Msg[8];
  end;
  TitleEditor.Memo.Text := Copy(Legend[i+1], 1, Pos('\n', Legend[i+1]) - 1);
  if (i <> 3) and (i <> 4) then
    begin
    Edit2.Text := FormatFloat('##0.0', LegendXshift[i+1]);
    Edit3.Text := FormatFloat('##0.0', LegendYshift[i+1]);
    end
  else
    begin
    Edit2.Text := FormatFloat('##0.0', LegendYshift[i+1]);
    Edit3.Text := FormatFloat('##0.0', LegendXshift[i+1]);
    end;
  if NewDS = 1 then
    FindMaxMin;
  {
  if (FloatToStr(Gmin[i]) = '0') and (FloatToStr(Gmax[i]) = '0') then FindMaxMin;
  if i <= 2 then
    if (Gmin[1] = Gmin[2]) and (Gmax[1] = Gmax[2]) then FindMaxMin else
  else
    if (Gmin[3] = Gmin[4]) and (Gmax[3] = Gmax[4]) then FindMaxMin;
}
  Edit4.Text := FloatToStr(Gmin[i]);
  Edit5.Text := FloatToStr(Gmax[i]);
  SpinEdit1.Value := Gdivision[i];
  SpinEdit2.Value := Gminordivision[i];
  Edit8.Text := FormatFloat('##0.0', GnumeralsXshift[i]);
  Edit9.Text := FormatFloat('##0.0', GnumeralsYshift[i]);
  if GridLine[i]  = 1 then CheckBox1.Checked := True else CheckBox1.Checked := False;
  if logarithm[i] = 1 then
  begin
    CheckBox2.Checked := True;
    SpinEdit2.Enabled := False;
    if Gminordivision[i] = -1 then CheckBox3.Checked := True;
  end
  else
  begin
    CheckBox2.Checked := False;
    SpinEdit2.Enabled := True;
  end;
  if Exponential[i] = 1 then
    chkExponential.Checked := True
  else
    chkExponential.Checked := False;
  CheckBox2Click(Sender);
  ShortDateFormat := DateFormat;
  if (i > 2) or (Gmax[i] < 1) or (Gmin[i] < 0) or (logarithm[i] = 1) then
  begin
    CheckBox4.Enabled := False;
  end
  else
  begin
    CheckBox4.Enabled := True;
    if OriginDay[i] > 0 then
    begin
   //   DateTimePicker.Text := DateToStr(OriginDay[i]);
      CheckBox4.Checked := True;
      DateTimePicker.Date := OriginDay[i];
    end
    else
      OriginDay[i] := Trunc(Now);
  end;
  DateTimePicker.Enabled := CheckBox4.Checked;
  edDateFormat.Enabled   := CheckBox4.Checked;
  lblDateFormat.Enabled  := CheckBox4.Checked;
  if NumVisible[i] = True then CheckBox5.Checked := True else CheckBox5.Checked := False;
  if i = 3 then
  begin
    CheckBox6.Visible := True;
    Panel3.Visible    := True;
    Button1.Visible   := True;
    CheckBox4.Top := 324;
    Panel3.Color := StripeColor;
    if (StripeColor <> BGcolor) and (StripeColor <> clWhite) then
      CheckBox6.Checked := True
    else
      CheckBox6.Checked := False;
    ChangeAllowed := True;
  end
  else
  begin
    CheckBox6.Visible := False;
    Panel3.Visible    := False;
    Button1.Visible   := False;
    CheckBox4.Top := 292;
  end;
  edDateFormat.Text := DateFormat;
  DateTimePicker.Top := CheckBox4.Top - 2;
  edDateFormat.Top := CheckBox4.Top + 30;
  lblDateFormat.Top := edDateFormat.Top + 2;
  Panel1.Top := edDateFormat.Top + 35;
  {$ifdef Linux}
  Height := Panel1.Top + 55;
  TitleEditor.Left := 20;
  edDateFormat.Left := lblDateFormat.Left + lblDateFormat.Width + 10;
  {$else}
  Height := Panel1.Top + 78;
  {$endif}
  TitleEditor.Enabled := not For_Math;
end;
//----------------------------------------------------------
procedure TAxisDialog.FindMaxMin;
var
  i, i2, i3, dumint, dumint1, dumint2: Integer;
  GX_, GY_  : Double;
begin
  i := AxisMode;
  Gmax[i] := -Power(10,10);
  Gmin[i] :=  Power(10,10);
  for i2 := 1 to NumDataSeries do
  begin
    if (AxisMode = 1) and (Xb[i2] = 0) then Continue;
    if (AxisMode = 2) and (Xt[i2] = 0) then Continue;
    if (AxisMode = 3) and (Yl[i2] = 0) then Continue;
    if (AxisMode = 4) and (Yr[i2] = 0) then Continue;
    for i3 := 1 to NumData[i2] do
      if AxisMode <= 2 then
      begin
        GX_ := GX[i2,i3]; //Xmult[i2] * GX[i2,i3] + Xoffset[i2];
        if GX_ <> 12345678910 then
        begin
          if GX_ > Gmax[i] then Gmax[i] := GX_;
          if GX_ < Gmin[i] then Gmin[i] := GX_;
        end;
      end
      else
      begin
        GY_ := GY[i2,i3]; //Ymult[i2] * GY[i2,i3] + Yoffset[i2];
        if GX[i2,i3] <> 12345678910 then
        begin
          if GY_ > Gmax[i] then Gmax[i] := GY_;
          if GY_ < Gmin[i] then Gmin[i] := GY_;
        end;
      end
  end;
  if AxisMode >= 3 then NewDs := 0;
  dumint := Floor(log10(Abs(Gmax[i] - Gmin[i])));
  Gmax[i] := Ceil (Gmax[i] * Power(10, - dumint)) / Power(10, - dumint);
  Gmin[i] := Floor(Gmin[i] * Power(10, - dumint)) / Power(10, - dumint);
  dumint1 := Floor(Gmax[i] * Power(10, - dumint));
  dumint2 := Floor(Gmin[i] * Power(10, - dumint));
  Gdivision[i] := dumint1 - dumint2;
  Case Gdivision[i] of
   8: Gdivision[i] := 4;
  10: Gdivision[i] := 5;
  end;
  Gminordivision[i] := 2;
end;
//----------------------------------------------------------
procedure TAxisDialog.Edit4Change(Sender: TObject);
begin
  Timer1.Enabled := True;
end;
//----------------------------------------------------------
procedure TAxisDialog.Timer1Timer(Sender: TObject);
var
  i, n: Integer;
  d, Gmax_, Gmin_ : Double;
begin
  i := AxisMode;
  if (Edit4.Text = '') or (Edit4.Text = '-') then
    Exit;
  if (Edit5.Text = '') or (Edit5.Text = '-') then
    Exit;
  if (Pos('E', Edit4.Text) > 1) and (StrToFloat(Edit4.Text) = 1) then
    Exit;
  if (Pos('E', Edit5.Text) > 1) and (StrToFloat(Edit5.Text) = 1) then
    Exit;
  Gmin_ := 0;
  Gmax_ := 0;
  try
    Gmin_ := StrToFloat(Edit4.Text);
    Gmax_ := StrToFloat(Edit5.Text);
    {
    if Gmax_*Gmin_ <= 0 then
      CheckBox1.Enabled := True
    else
      begin
      CheckBox1.Enabled := False;
      GridLine[i] := 0;
      end;
    }
    if Gmax_ = Gmin_ then
      Exit;
    if (Gmax_ <= 0) or (Gmin_ <= 0) then
    begin
      logarithm[i] := 0;
      CheckBox2.Checked := False;
      CheckBox2.Enabled := False;
    end
    else
      CheckBox2.Enabled := True;
  except
    ;
  end;
  Gdivision[i] := SpinEdit1.Value;
  if CheckBox2.Checked then
  begin
    Gmin_ := IntPower(10, Floor(log10(Gmin_ * 1.01)));
    Gmax_ := IntPower(10, Ceil(log10(Gmax_ * 0.99)));
    Edit4.Text := FloatToStr(Gmin_);
    Edit5.Text := FloatToStr(Gmax_);
    n := Gdivision[i];
    Gdivision[i] := Abs(Ceil(log10(Gmax_ * 0.99)) - Floor(log10(Gmin_*1.01)));
    if 2 * n = Gdivision[i] then
    begin
      Gdivision[i] := n;
      CheckBox3.Checked := False;
    end;
  end
  else
  begin
    d := Abs(Gmax_ - Gmin_);
    if d > 0 then
    begin
      d := 100 * d * Power(10, Trunc(log10(d)));
   //   ShowMessage(IntToStr((Trunc(d) mod Gdivision[i])));
      while (Gdivision[i] > 2) and ((Trunc(d + 0.0000000000001) mod Gdivision[i]) > 0) do
        Dec(Gdivision[i]);
    end;
  end;
  SpinEdit1.Value := Gdivision[i];
  if Abs(log10(Abs(Gmin_ - Gmax_) / 2)) < 4 then
  begin
    chkExponential.Checked := False;
    Exponential[i] := 0
  end
  else
  begin
    chkExponential.Checked := True;
    Exponential[i] := 1;
  end;
  Timer1.Enabled := False;
end;
//----------------------------------------------------------
procedure TAxisDialog.OKClick(Sender: TObject);
var i : Integer;
begin
  i := AxisMode;
  if TitleEditor.Memo.Text = '' then
    TitleEditor.Memo.Text := '   ';
  Legend[i+1] := TitleEditor.Memo.Text + '\n';
  if (i <> 3) and (i <> 4) then
  begin
    LegendXshift[i+1] := StrToFloat(Edit2.Text);
    LegendYshift[i+1] := StrToFloat(Edit3.Text);
  end
  else
  begin
    LegendYshift[i+1] := StrToFloat(Edit2.Text);
    LegendXshift[i+1] := StrToFloat(Edit3.Text);
  end;
  GnumeralsXshift[i] := StrToFloat(Edit8.Text);
  GnumeralsYshift[i] := StrToFloat(Edit9.Text);
  Gdivision[i]       := StrToInt(SpinEdit1.Text);
  Gminordivision[i]  := StrToInt(SpinEdit2.Text);
  if CheckBox1.Checked = True then
    if i < 3 then
    begin
      GridLine[1]  := 1;
      GridLine[2]  := 1;
    end
    else
    begin
      GridLine[3]  := 1;
      GridLine[4]  := 1;
    end
  else
    if i < 3 then
    begin
      GridLine[1]  := 0;
      GridLine[2]  := 0;
    end
    else
    begin
      GridLine[3]  := 0;
      GridLine[4]  := 0;
    end;
  if CheckBox2.Checked = True then
    logarithm[i] := 1
  else
    logarithm[i] := 0;
  if logarithm[i] = 1 then
    if CheckBox3.Checked = True then
      Gminordivision[i] := -1
    else
      Gminordivision[i] := 0;
  Gmin[i] := StrToFloat(Edit4.Text);
  Gmax[i] := StrToFloat(Edit5.Text);
  if Gmin[i] = Gmax[i] then
  begin
    Gmin[i] := 0;
    if Gmax[i] = 0 then
      Gmax[i] := 100;
  end;
  if logarithm[i] = 1 then
  begin
    Gmin[i] := IntPower(10, Floor(log10(Gmin[i]*1.01)));
    Gmax[i] := IntPower(10, Ceil(log10(Gmax[i]*0.99)));
  end;
  if i <= 2 then
    if CheckBox4.Checked = True then
    //  OriginDay[i] := Trunc(StrToDate(DateTimePicker.Text))
      OriginDay[i] := Trunc(DateTimePicker.Date)
    else
      OriginDay[i] := 0;
  NumVisible[i] := CheckBox5.Checked;
  if i = 3 then
    StripeColor := Panel3.Color;
  AllowWrite := True;
  Main.WriteLegend(Main.GraphImage.Canvas, asc, i, i);
  if chkExponential.Checked then
    Exponential[i] := 1
  else
    Exponential[i] := 0;
  Modified[NumFigure] := 1;
  DateFormat := edDateFormat.Text;
  SetDateSeparator(DateFormat);
end;

Procedure TAxisDialog.SetDateSeparator(S: String);
begin
  if Pos('/', S) > 0 then
    DateSeparator := '/'
  else
    if Pos('-', S) > 0 then
      DateSeparator := '-'
    else
      if Pos('.', S) > 0 then
        DateSeparator := '.'
end;

//----------------------------------------------------------
procedure TAxisDialog.CheckBox2Click(Sender: TObject);
begin
  if CheckBox2.Checked = True then
  begin
    Label7.Visible  := False;
    SpinEdit2.Visible   := False;
    CheckBox3.Visible := True;
    Timer1Timer(Sender);
  end
  else
  begin
    Label7.Visible  := True;
    SpinEdit2.Visible   := True;
    CheckBox3.Visible := False;
  end;
end;
//===============================================================================================================================

procedure TAxisDialog.CheckBox4Click(Sender: TObject);
begin
  DateTimePicker.Enabled := CheckBox4.Checked;
  edDateFormat.Enabled   := CheckBox4.Checked;
  lblDateFormat.Enabled  := CheckBox4.Checked;
end;

procedure TAxisDialog.TitleEditorEnter(Sender: TObject);
begin
  TitleEditor.DefaultFont := Main.ToolBar.Font;// .Image1.Canvas.Font;
  TitleEditor.DefaultFont.Color := clBlack;
  TitleEditor.HyperText.Shadow := False;
//  TitleEditor.ToolBar.Font.Name := Main.ToolBar.Font.Name;
//  TitleEditor.ToolBar.Font.Size := Ceil(ToolBar.Font.Size * mag);
  PopupMenu := TitleEditor.PopupMenu;
  TitleEditor.FrameEnter(Sender);
end;

procedure TAxisDialog.Button1Click(Sender: TObject);
begin
  ColorDialog1.Color := StripeColor;
  if ColorDialog1.Execute then
  begin
    StripeColor := ColorDialog1.Color;
    Panel3.Color := ColorDialog1.Color;
    if StripeColor = BGcolor then
      CheckBox1.Checked := False
    else
    begin
      ChangeAllowed := False;
      CheckBox6.Checked := True;
      SpinEdit2.Value := 1;
      Gminordivision[4] := 1;
      ChangeAllowed := True;
    end;
  end;
 // ShowMessage(IntToStr(Panel3.Color));
  Panel3Exit(Sender);
end;

procedure TAxisDialog.Panel3Exit(Sender: TObject);
begin
  With Image1.Canvas do
  begin
    Pen.Color := Panel3.Color;
    Brush.Color := Panel3.Color;
    Rectangle(0, 0, Image1.Width, Image1.Height);
  end;
end;

procedure TAxisDialog.Panel3Enter(Sender: TObject);
begin
  With Image1.Canvas do
  begin
    Pen.Color := clYellow;
    Pen.Style := psDot;
    Brush.Color := Panel3.Color;
    Rectangle(0, 0, Image1.Width, Image1.Height);
  end;
end;

procedure TAxisDialog.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Button1Click(Sender)
end;

procedure TAxisDialog.CheckBox6Click(Sender: TObject);
begin
  if ChangeAllowed then
    if CheckBox6.Checked then
      Button1Click(Sender)
    else
    begin
      Panel3.Color := BGcolor;
      Panel3Exit(Sender);
    end;
end;

procedure TAxisDialog.HelpButtonClick(Sender: TObject);
{$ifdef Linux}
//var S: String;
{$endif}
begin
  {$ifdef Linux}
  //S := '';
  //QKeyEvent_create(QEventType_KeyPress, Key_F1, 0, 0, @S, False, 0);
  InvokeHelp;
  //Main.IndexClick(Sender);
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
 // Application.HelpCommand(HELP_CONTEXT, 0);
end;

procedure TAxisDialog.TitleEditorMemoExit(Sender: TObject);
begin
  PopupMenu := nil;
end;

procedure TAxisDialog.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not CheckBox4.Checked then
    if AxisMode < 3 then
      OriginDay[AxisMode] := 0;
end;

procedure TAxisDialog.SpinEdit1Changed(Sender: TObject; NewValue: Integer);
begin
  Timer1.Enabled := True;
end;

procedure TAxisDialog.SpinEdit1Change(Sender: TObject);
begin
  Timer1.Enabled := True;
end;

end.
