unit ContourMap;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QForms, QControls, Variants, QGraphics, QDialogs, QStdCtrls, Types, Math;
{$else}
  SysUtils, Classes, Forms, Controls, Variants, Graphics, Dialogs, StdCtrls,
  Types, Printers, Windows, Math;
{$endif}

type
  TTriangle  = array [1..3] of TPoint;
  TNodeValue = array [1..3] of Double;

  { TContourMap }

  TContourMap = class(TComponent)

  private
    FTargetCanvas: TCanvas;
    FMagPrint, FMagScreen, FDisplaySize: Double;
    FNumElement, FNumNode, FNumArrow, FTop, FLeft: Integer;
    mag: Single;
    NX, NY: Integer;
    procedure SetAdjuster;
  public
    x, y, v: array[1..10000] of Extended;        // starting from 0 is for compatibility to C++Builder
    Xv, Yv, Vx, Vy: array[1..1000] of Single;
    Vertex: array[1..20000, 1..3] of Integer;
    OutLineX, OutLineY: array[0..200] of Single;
    constructor Create(AOwner: TComponent); override;
    procedure GenerateGrid(NumX, NumY: Integer);
    procedure GenerateMesh;
    procedure SetConstantIncrementXvalues(RectWidth: Double);
    procedure SetConstantIncrementYvalues(RectHeight: Double);
    procedure DrawContour(Color: TColor; Max, Min, Step: Single; LineWidth: Integer = 1);
    procedure DrawShadingMap(Color: TColor; Max, Min: Single);
    procedure DrawElement(Color_: TColor);
    procedure DrawOutLine(Color_: TColor; NumOutLineNode: Integer);
    procedure DrawArrow(Angle, HeadLength: Single; Color_: TColor; PaintHead: Boolean);
  published
    property MagPrint: Double read FMagPrint write FMagPrint;
    property MagScreen: Double read FMagScreen write FMagScreen;
    property Top: Integer read FTop write FTop;
    property Left: Integer read FLeft write FLeft;
    property NumNode: Integer read FNumNode write FNumNode;
    property NumElement: Integer read FNumElement write FNumElement;
    property NumArrow: Integer read FNumArrow write FNumArrow;
    property DisplaySize: Double read FDisplaySize write FDisplaySize;
    property TargetCanvas: TCanvas read FTargetCanvas write FTargetCanvas;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Misc', [TContourMap]);
end;

constructor TContourMap.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  MagPrint := 1;
  MagScreen := 1;
  DisplaySize := 14;
  Left := 0;
  Top  := 0;
 // TargetCanvas := Printer.Canvas;
end;

procedure TContourMap.GenerateGrid(NumX, NumY: Integer);
var i, j, k, m: Integer;
begin
  NX := NumX;
  NY := NumY;
  NumNode := NX * NY;
  NumElement := 2 * (NX - 1) * (NY - 1);
  for i := 1 to NX - 1 do
    for j := 1 to NY - 1 do
    begin
      k := (j - 1) * (NX - 1) + i;
      Vertex[k,1] := k + (j - 1);      //  1-2
      Vertex[k,2] := Vertex[k,1] + 1;  //  |/
      Vertex[k,3] := Vertex[k,1] + NX; //  3
      m := (NX - 1) * (NY - 1) + k;
      Vertex[m,1] := Vertex[k,2];      //     1
      Vertex[m,2] := Vertex[k,3];      //    /|
      Vertex[m,3] := Vertex[m,2] + 1;  //   2-3
    end;
end;

procedure TContourMap.GenerateMesh;
  var
    Number, SegStart, SegEnd: array[1..1000] of Integer;
    Distance, Xcenter, Ycenter: array[1..300] of Double;
    StoredDistance: array[1..200, 1..200] of Double;
    i, ii, j, k, n, NumSegment: Integer;
    x11, x12, x21, x22, y11, y12, y21, y22, a1, a2, x_, y_: Double;
    Flag: Boolean;

  procedure Sort2;
  var
    sign, i, j, k, dumx: Integer;
    dumy: Double;
  begin
    sign := 1;
    for i := 1 to NumNode do
    begin
      Number[i] := i;
      for j := 1 to i - 1 do
        if sign * Distance[i] < sign * Distance[j] then
        begin
          dumy := Distance[i];
          dumx := Number[i];
          for k := i - 1 downto j do
          begin
            Distance[k+1] := Distance[k];
            Number[k+1]   := Number[k];
          end;
          Distance[j] := dumy;
          Number[j] := dumx;
        end;
    end;
    for i := 1 to NumNode -1 do
      if Distance[i+1] <= Distance[i] + 1 then
      begin
        dumx := Number[i];
        Number[i] := Number[i+1];
        Number[i+1] := dumx;
      end;
  end;

  function IsCrossed(OnBorder: Boolean = False) : Boolean;
    function IsBracketed(z, z1, z2: Double): Boolean;
    var Er: Double;
    begin
      Result := False;
      Er := 1E-11;
      if (z > z1 - 0.000001) and (z < z1 + 0.000001) and (z > z2 - 0.000001) and (z < z2 + 0.000001) then
        Result := True
      else
        if OnBorder then
        begin
          if z1 < z2 then
          begin
            if (z1 <= z + Er) and (z <= z2 + Er) then
              Result := True
          end
          else
            if (z2 <= z + Er) and (z <= z1 + Er) then
              Result := True;
        end
        else
        begin
          if z1 < z2 then
          begin
            if (z1 < z - Er) and (z < z2 - Er) then
              Result := True
          end
          else
            if (z2 < z - Er) and (z < z1 - Er) then
              Result := True;
        end;
    end;
  begin
    if x12 <> x11 then
      a1 := (y12 - y11) / (x12 - x11)
    else
      a1 := 123456789;
    if x22 <> x21 then
    begin
      a2 := (y22 - y21) / (x22 - x21);
      if a1 <> a2 then
        x_ := (a1 * x11 - a2 * x21 + y21 - y11) / (a1 - a2)
      else
      begin
        //Crossed := False;
        Result  := False;
        Exit;
      end
    end
    else
      x_ := x21;
    if a1 <> 123456789 then
      y_ := a1 * (x_ - x11) + y11
    else
    begin
      x_ := x11;
      y_ := a2 * (x_ - x21) + y21
    end;
    if (IsBracketed(x_, x11, x12)) and (IsBracketed(x_, x21, x22)) and (IsBracketed(y_, y11, y12)) and (IsBracketed(y_, y21, y22)) then
      Result := True
    else
      Result := False;
  end;

  function CheckCross(StartPos, EndPos: Integer): Boolean;
  var
    k, count: Integer;
  begin
    Result := False;
    count := 0;
    if NumSegment > 0 then
      for k := 1 to NumSegment do
      begin
        x21 := x[SegStart[k]];
        x22 := x[SegEnd[k]];
        y21 := y[SegStart[k]];
        y22 := y[SegEnd[k]];
        if ((StartPos = SegStart[k]) and (EndPos = SegEnd[k])) or ((StartPos = SegEnd[k]) and (EndPos = SegStart[k])) then
          Inc(count);
        if (count = 2) or (IsCrossed) then
        begin
       //   Crossed := True;
          Result := True;
          Exit;
        end;
      end;
  end;

  function CheckInclusion: Boolean;
  var
    i: Integer;

    function IsIncluded(i: Integer): Boolean;
    var
      j: Integer;
      Crossed: Boolean;
    begin
      Crossed := False;
      for j := 1 to 3 do
      begin
        x21 := X[Vertex[i,j]];
        y21 := Y[Vertex[i,j]];
        if j < 3 then
        begin
          x22 := X[Vertex[i,j+1]];
          y22 := Y[Vertex[i,j+1]];
        end
        else
        begin
          x22 := X[Vertex[i,1]];
          y22 := Y[Vertex[i,1]];
        end;
        if IsCrossed(True) then
          Crossed := True;
      end;
      Result := not Crossed;
    end;

  begin
    Result := False;
    x12 := Xcenter[n];
    y12 := Ycenter[n];
    for i := 1 to n - 1 do
    begin
      if (Xcenter[i] = x12) and (Ycenter[i] = Y12) then  // duplicate
      begin
        Result := True;
        Exit;
      end;
     // Continue;
      x11 := Xcenter[i];
      y11 := Ycenter[i];
      if (IsIncluded(i)) or (IsIncluded(n)) then
      begin
        Result := True;
        Exit;
      end;
    end;
  end;

  function Area: Double;
  var
    aX, aY, bX, bY: Double;
  begin
    aX := X[Vertex[n,2]] - X[Vertex[n,1]];
    aY := Y[Vertex[n,2]] - Y[Vertex[n,1]];
    bX := X[Vertex[n,3]] - X[Vertex[n,1]];
    bY := Y[Vertex[n,3]] - Y[Vertex[n,1]];
    Result := ((Sqr(aX) + Sqr(aY)) * (Sqr(bX) + Sqr(bY)) - Sqr(aX * bX + aY * bY)); //0.5 * sqrt
    // actually, this is 2A^2
  end;

begin
  NumSegment := 0;
  n := 0;
  repeat
    Inc(n);
  until y[n] <> y[n+1];
  if (y[NumNode-n+1] = y[NumNode]) and (x[n] = x[NumNode]) then
    GenerateGrid(n, NumNode div n)
  else
  begin
    if NumNode > 200 then
    begin
      ShowMessage('Sorry, number of node must be less than 200.');
      Exit;
    end;
    for i := 1 to 1000 do
    begin
      SegStart[i] := 0;
      SegEnd[i]   := 0;
    end;
    for i := 1 to 200 do
      for j := 1 to 200 do
        StoredDistance[i,j] := -1;
    n := 1;
    for ii := 1 to 3 do //3
    begin
      if (ii mod 2 = 1) then
        i := 0
      else
        i := Numnode + 1;
      repeat
        if (ii mod 2 = 1) then
          Inc(i)
        else
          Dec(i);
        Vertex[n,1] := i;
        for j := 1 to NumNode do
          if StoredDistance[i,j] = -1 then
          begin
            Distance[j] := (Sqr(x[i] - x[j]) + Sqr(y[i] - y[j]));  // Sqrt
            StoredDistance[i,j] := Distance[j];
            StoredDistance[j,i] := Distance[j];
          end
          else
            Distance[j] := StoredDistance[i,j];
        Sort2;
        j := 1;
        repeat
          inc(j);
          if (i <> Number[j]) then
          begin
         //   Crossed := False;
            x11 := x[i];
            x12 := x[Number[j]];
            y11 := y[i];
            y12 := y[Number[j]];
            if CheckCross(i, Number[j]) = False then
            begin
              Vertex[n,2] := Number[j];
              k := j; // 1 ?? j??
              repeat
                Inc(k);
                Flag := False;
                if (Number[j] <> Number[k]) and (i <> Number[k]) then
                begin
                  x11 := x[i];
                  x12 := x[Number[k]];
                  y11 := y[i];
                  y12 := y[Number[k]];
                  if CheckCross(i, Number[k]) = False then
                  begin
                    Vertex[n,3] := Number[k];
                    x11 := x[Vertex[n,2]];
                    x12 := x[Vertex[n,3]];
                    y11 := y[Vertex[n,2]];
                    y12 := y[Vertex[n,3]];
                    Xcenter[n] := (X[Vertex[n,1]] + X[Vertex[n,2]] + X[Vertex[n,3]]) / 3;
                    Ycenter[n] := (Y[Vertex[n,1]] + Y[Vertex[n,2]] + Y[Vertex[n,3]]) / 3;
                    // Second criteria is to exclude very thin triangle. fine tuning is required for the coefficient
                    if CheckCross(Vertex[n,2], Vertex[n,3]) = False then
                      if Area > 0.001 * Sqr(Distance[k]) then
                        if CheckInclusion = False then
                        begin
                          SegStart[NumSegment+1]   := Vertex[n,1];
                          SegStart[NumSegment+2]   := Vertex[n,1];
                          SegStart[NumSegment+3]   := Vertex[n,2];
                          SegEnd[NumSegment+1]     := Vertex[n,2];
                          SegEnd[NumSegment+2]     := Vertex[n,3];
                          SegEnd[NumSegment+3]     := Vertex[n,3];
                          NumSegment := NumSegment + 3;
                          n := n + 1;
                          Flag := True;
                          break;
                        end;
                  end;
                end;
              until k >= Min(NumNode, 40);// div 2;
              if Flag then
                break;
            end; // if
          end; // if
        until j = Min(NumNode div 2, 20);// - 1;
      until ((ii mod 2 = 1) and (i = NumNode)) or ((ii mod 2 = 0) and (i = 1));
    end;
    NumElement := n - 1;
  end;
end;

procedure TContourMap.SetConstantIncrementXvalues(RectWidth: Double);
var
  i: Integer;
  dx: Double;
begin
  dx := RectWidth / NX;
  for i := 0 to NumNode do   // starting from 0 is for compatibility to C++Builder
    x[i] := dx * (i - NX * Trunc(i / NX - 0.00001) - 0.5);
end;

procedure TContourMap.SetConstantIncrementYvalues(RectHeight: Double);
var
  i: Integer;
  dy: Double;
begin
  dy := RectHeight / NY;
  for i := 0 to NumNode do
    y[i] := dy * (Trunc(i / NX - 0.00001) + 0.5);
end;

procedure TContourMap.DrawElement(Color_: TColor);
var i: Integer;
begin
  SetAdjuster;
  with TargetCanvas do
  begin
    Pen.Color := Color_;
    for i := 1 to NumElement do
    begin
      MoveTo(Left + Trunc(x[Vertex[i,1]] * mag), Top + Trunc(y[Vertex[i,1]] * mag));
      LineTo(Left + Trunc(x[Vertex[i,2]] * mag), Top + Trunc(y[Vertex[i,2]] * mag));
      LineTo(Left + Trunc(x[Vertex[i,3]] * mag), Top + Trunc(y[Vertex[i,3]] * mag));
      LineTo(Left + Trunc(x[Vertex[i,1]] * mag), Top + Trunc(y[Vertex[i,1]] * mag));
    end;
  end;
end;

procedure TContourMap.DrawShadingMap(Color: TColor; Max, Min: Single);
var
  i, j : Integer;
  sva, svb, svc: Double;
  Tr: TTriangle;
  zo: TNodeValue;
  //a, b: TPoint;
  TotalArea, aX, aY, bX, bY: Single;

  procedure PaintTriangle(Tr: TTriangle; z: TNodeValue ; MinimumArea: Single);
  var
    AVG, Area: Single;
    a, b, c: TPoint;
    p1, p2, p3: ShortInt;
    NextTr: TTriangle;
    NextZ: TNodeValue;

    function GetLighterColor(ratio: Single): Integer;
    var
      dumstr : String;
      Red, Blue, Green : Integer;
    begin
      dumstr := IntToHex(Color, 6);
      Blue  := 255 - Trunc((255 - StrToInt('$' + Copy(dumstr, 1, 2))) * ratio);
      Green := 255 - Trunc((255 - StrToInt('$' + Copy(dumstr, 3, 2))) * ratio);
      Red   := 255 - Trunc((255 - StrToInt('$' + Copy(dumstr, 5, 2))) * ratio);
      Result := Blue * 256 * 256 + Green * 256 + Red;
    end;

    function SquareVector(v: TPoint): Integer;
    begin
      Result := v.X * v.X + v.Y * v.Y;
    end;
  begin
    //MinimumArea := 2 * Sqr(MinimumArea);
    a.X := Tr[2].X - Tr[1].X;
    a.Y := Tr[2].Y - Tr[1].Y;
    b.X := Tr[3].X - Tr[1].X;
    b.Y := Tr[3].Y - Tr[1].Y;
    c.X := b.X - a.X;
    c.Y := b.Y - a.Y;
    sva := SquareVector(a);
    svb := SquareVector(b);
    svc := SquareVector(c);
    Area := sva * svb;
    Area := Area - Sqr(a.X * b.X + a.Y * b.Y);
    if Area > 0 then
      Area := 0.5 * Sqrt(Area);
    if Area < MinimumArea then
      with TargetCanvas do
      begin
        Pen.Color := Color;
        AVG := 0.33 * (z[1] + z[2] + z[3]);
        if AVG < Min then
          Brush.Color := clWhite
        else
          if AVG < Max then
            Brush.Color := GetLighterColor((AVG - Min) / (Max - Min))
          else
            Brush.Color := Color;
        Pen.Color   := Brush.Color;
        Polygon([Tr[1], Tr[2], Tr[3]]);
      end
    else
    begin
      {
      if z[1] > z[2] then
      begin
        if z[2] > z[3] then
        begin
          p1 := 1;
          p2 := 2;
          p3 := 3;
        end
        else
        begin
          if z[1] < z[3] then
          begin
            p1 := 2;
            p2 := 3;
            p3 := 1;
          end
          else
          begin
            p1 := 1;
            p2 := 3;
            p3 := 2;
          end;
        end;
      end
      else
        if z[2] < z[3] then
        begin
          p1 := 3;
          p2 := 2;
          p3 := 1;
        end
        else
        begin
          if z[1] > z[3] then
          begin
            p1 := 2;
            p2 := 1;
            p3 := 3;
          end
          else
          begin
            p1 := 3;
            p2 := 1;
            p3 := 2;
          end;
        end;
      p1 := 1;
      p2 := 2;
      p3 := 3;
      }
      if sva > svb then
      begin
        if sva > svc then
        begin
          p1 := 1;
          p2 := 3;
          p3 := 2;
        end
        else
        begin
          if svb > svc then
          begin
            p1 := 1; //svb (1&3) is the longest
            p2 := 2;
            p3 := 3;
          end
          else
          begin
            p1 := 2; //svc (2&3) is the longest
            p2 := 1;
            p3 := 3;
          end;
        end
      end
      else
      begin
        if svb < svc then
        begin
          p1 := 2;  //svc (2&3) is the longest
          p2 := 1;
          p3 := 3;
        end
        else
        begin
          p1 := 1;  //svb (1&3) is the longest
          p2 := 2;
          p3 := 3;
        end
      end;

      NextTr[1].X := (Tr[p1].X + Tr[p3].X) div 2;
      NextTr[1].Y := (Tr[p1].Y + Tr[p3].Y) div 2;
      NextZ[1]    := (z[p1] + z[p3]) / 2;
      NextTr[2].X := Tr[p1].X;
      NextTr[2].Y := Tr[p1].Y;
      NextZ[2]    := z[p1];
      NextTr[3].X := Tr[p2].X;
      NextTr[3].Y := Tr[p2].Y;
      NextZ[3]    := z[p2];
      PaintTriangle(NextTr, NextZ, MinimumArea);
      NextTr[2].X := Tr[p3].X;
      NextTr[2].Y := Tr[p3].Y;
      NextZ[2]    := z[p3];
      PaintTriangle(NextTr, NextZ, MinimumArea);
    end;
  end;

begin
  SetAdjuster;
  TotalArea := 0;
  for i := 1 to NumElement do
  begin
    for j := 1 to 3 do
    begin
      Tr[j].X := Left + Trunc(x[Vertex[i,j]] * mag);
      Tr[j].Y := Top  + Trunc(y[Vertex[i,j]] * mag);
    end;
    aX := Tr[2].X - Tr[1].X;
    aY := Tr[2].Y - Tr[1].Y;
    bX := Tr[3].X - Tr[1].X;
    bY := Tr[3].Y - Tr[1].Y;
    TotalArea := TotalArea + 0.5 * Sqrt((aX * aX + aY * aY) * (bX * bX + bY * bY) - Sqr(aX * bX + aY * bY));
  end;
  for i := 1 to NumElement do
  begin
    for j := 1 to 3 do
    begin
      Tr[j].X := Left + Trunc(x[Vertex[i,j]] * mag);
      Tr[j].Y := Top  + Trunc(y[Vertex[i,j]] * mag);
      zo[j] := v[Vertex[i,j]];
    end;
    //aX := Tr[2].X - Tr[1].X;
    //aY := Tr[2].Y - Tr[1].Y;
    //bX := Tr[3].X - Tr[1].X;
    //bY := Tr[3].Y - Tr[1].Y;
   // OriginalArea := 0.5 * Sqrt((aX * aX + aY * aY) * (bX * bX + bY * bY) - Sqr(aX * bX + aY * bY));
    PaintTriangle(Tr, zo, TotalArea * 0.0005);// 0.05 * OriginalArea);
  end;
end;

procedure TContourMap.DrawContour(Color: TColor; Max, Min, Step: Single; LineWidth: Integer = 1);
var
  i, j: Integer;
  Val: Double;
  xc, yc:array[1..3] of Double;
begin
  SetAdjuster;
  Val := Max * 1.0001;
  TargetCanvas.Pen.Color := Color;
  TargetCanvas.Pen.Width := LineWidth;
  repeat
    for i := 1 to NumElement do
    begin
      j := 1;
      //if Val > Max * 1.0001 then
      //  ShowMessage(IntToStr(i));
      if ((v[Vertex[i,1]] >= Val) and (v[Vertex[i,2]] < Val)) or ((v[Vertex[i,1]] <= Val) and (v[Vertex[i,2]] > Val)) then
      begin
        if x[Vertex[i,1]] <> x[Vertex[i,2]] then
          xc[j] := (Val - v[Vertex[i,1]]) / (v[Vertex[i,2]] - v[Vertex[i,1]]) * (x[Vertex[i,2]] - x[Vertex[i,1]]) + x[Vertex[i,1]]
        else
          xc[j] := x[Vertex[i,1]];
        if y[Vertex[i,1]] <> y[Vertex[i,2]] then
          yc[j] := (Val - v[Vertex[i,1]]) / (v[Vertex[i,2]] - v[Vertex[i,1]]) * (y[Vertex[i,2]] - y[Vertex[i,1]]) + y[Vertex[i,1]]
        else
          yc[j] := y[Vertex[i,1]];
        j := j + 1;
      end;
      if ((v[Vertex[i,1]] >= Val) and (v[Vertex[i,3]] < Val)) or ((v[Vertex[i,1]] <= Val) and (v[Vertex[i,3]] > Val)) then
      begin
        if x[Vertex[i,1]] <> x[Vertex[i,3]] then
          xc[j] := (Val - v[Vertex[i,1]]) / (v[Vertex[i,3]] - v[Vertex[i,1]]) * (x[Vertex[i,3]] - x[Vertex[i,1]]) + x[Vertex[i,1]]
        else
          xc[j] := x[Vertex[i,1]];
        if y[Vertex[i,1]] <> y[Vertex[i,3]] then
          yc[j] := (Val - v[Vertex[i,1]]) / (v[Vertex[i,3]] - v[Vertex[i,1]]) * (y[Vertex[i,3]] - y[Vertex[i,1]]) + y[Vertex[i,1]]
        else
          yc[j] := y[Vertex[i,1]];
        j := j + 1;
      end;
      if (j = 2) or (xc[1] = xc[2]) then
        if ((v[Vertex[i,2]] >= Val) and (v[Vertex[i,3]] < Val)) or ((v[Vertex[i,2]] <= Val) and (v[Vertex[i,3]] > Val)) then
        begin
          if x[Vertex[i,2]] <> x[Vertex[i,3]] then
            xc[j] := (Val - v[Vertex[i,2]]) / (v[Vertex[i,3]] - v[Vertex[i,2]]) * (x[Vertex[i,3]] - x[Vertex[i,2]]) + x[Vertex[i,2]]
          else
            xc[j] := x[Vertex[i,2]];
          if y[Vertex[i,2]] <> y[Vertex[i,3]] then
            yc[j] := (Val - v[Vertex[i,2]]) / (v[Vertex[i,3]] - v[Vertex[i,2]]) * (y[Vertex[i,3]] - y[Vertex[i,2]]) + y[Vertex[i,2]]
          else
            yc[j] := y[Vertex[i,2]];
          j := j + 1;
        end;
      if j = 3 then
      begin
        TargetCanvas.MoveTo(Left + Trunc(xc[1] * mag), Top + Trunc(yc[1] * mag));
        TargetCanvas.LineTo(Left + Trunc(xc[2] * mag), Top + Trunc(yc[2] * mag));
      end;
    end;
    Val := Val - Step;
  until Val < Min;
  TargetCanvas.Pen.Width := 1;
end;

procedure TContourMap.SetAdjuster;
{$ifndef Linux}
//var a0: Single;
{$endif}
begin
  (*
  if TargetCanvas = Printer.Canvas then
  begin
  {$ifdef Linux}
    mag := MagPrint;
  {$else}
    a0  := Screen.Height / (0.6 * 25.4 * DisplaySize);
    mag := MagPrint * 14.2 * GetDeviceCaps(Printer.Handle, LOGPIXELSX) / 360 / a0;
  {$endif}
  end
  else
  *)
  begin
    mag := MagScreen;
  end;
end;

procedure TContourMap.DrawOutLine(Color_: TColor; NumOutLineNode: Integer);
var i: Integer;
begin
  SetAdjuster;
  with TargetCanvas do
  begin
    Pen.Color := Color_;
    for i := 1 to NumOutLineNode - 1 do
    begin
      MoveTo(Left + Trunc(OutLineX[i] * mag)  , Top + Trunc(OutLineY[i] * mag));
      LineTo(Left + Trunc(OutLineX[i+1] * mag), Top + Trunc(OutLineY[i+1] * mag));
    end;
  end;
end;

procedure  TContourMap.DrawArrow(Angle, HeadLength: Single; Color_: TColor; PaintHead: Boolean);
var
  i, j: Integer;
  v0, vh1, vh2: TPoint;
  Ph: Array[1..3] of TPoint;
  P1, P2: array[1..1000] of TPoint;
  procedure DrawArrowHead(X1, Y1, X2, Y2: Single);
    function Roundoff(X: Extended): Longint;
    begin
      if x >= 0 then Result := Trunc(x + 0.5)
      else Result := Trunc(x - 0.5);
    end;

    function ResizeVector(v:TPoint; len: Double): TPoint;
    var l: Double;
    begin
      l := Sqrt(Sqr(v.x) + Sqr(v.y));
      Result.x := Roundoff(v.x * len / l);
      Result.y := Roundoff(v.y * len / l);
    end;

    function RotateVector(v:TPoint; arg: Double): TPoint;
    begin
      Result.x := Roundoff(Cos(arg) * v.x - Sin(arg) * v.y);
      Result.y := Roundoff(Sin(arg) * v.x + Cos(arg) * v.y);
    end;
  begin
    with TargetCanvas do
    begin
      Pen.Color   := Color_;
      Brush.Color := Color_;
      P1[i].x := Trunc(X1);
      P1[i].y := Trunc(Y1);
      P2[i].x := Trunc(X2);
      P2[i].y := Trunc(Y2);
      v0.x := -P2[i].x + P1[i].x;
      v0.y := -P2[i].y + P1[i].y;
      if (v0.x <> 0) or (v0.y <> 0) then
      begin
        if Sqrt(Sqr(v0.x) + Sqr(v0.y)) < HeadLength * mag then
          j := Trunc(Sqrt(Sqr(v0.x) + Sqr(v0.y)))
        else
          j := Trunc(HeadLength * mag);
        vh1 := ResizeVector(RotateVector(v0, 2 * 3.14 * Angle / 360), j);
        vh2 := ResizeVector(RotateVector(v0,-2 * 3.14 * Angle / 360), j);
        MoveTo(P1[i].x, P1[i].y);
        LineTo(P2[i].x, P2[i].y);
        if PaintHead = False then
        begin
          LineTo(P2[i].x + vh1.x, P2[i].y + vh1.y);
          MoveTo(P2[i].x        , P2[i].y);
          LineTo(P2[i].x + vh2.x, P2[i].y + vh2.y);
        end
        else
        begin
          Ph[1]   := P2[i];
          Ph[2].x := P2[i].x + vh1.x;
          Ph[2].y := P2[i].y + vh1.y;
          Ph[3].x := P2[i].x + vh2.x;
          Ph[3].y := P2[i].y + vh2.y;
         // Polygon([Point(P1[i].x, P1[i].y), Point(P2[i].x + vh1.x, P2[i].y + vh1.y), Point(P2[i].x + vh2.x, P2[i].y + vh2.y)]);
          TargetCanvas.Polygon(Ph);
        end;
      end;
    end;
  end;
begin
  for i := 1 to NumArrow do
    DrawArrowHead(Left + Xv[i] * mag, Top + Yv[i] * mag, Left + (Xv[i] + Vx[i]) * mag, Top + (Yv[i] + Vy[i]) * mag);
end;
end.
