unit Integral;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QClipbrd, Qt, IniFiles
  ;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Clipbrd, IniFiles;
{$endif}

type
  glsarray = array [1..3000] of Double;
  TIntegralDialog = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Panel2: TPanel;
    OK: TButton;
    CalcButton: TButton;
    HelpButton: TButton;
    Label3: TLabel;
    btnCopy: TButton;
    chkPaint: TCheckBox;
    edLB: TEdit;
    edUB: TEdit;
    edResult: TEdit;
    procedure FormShow(Sender: TObject);
    procedure CalcButtonClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
  public
    TheValue: Double; // d: dummy
    procedure Calculate;
    procedure SortTwo(n: integer;var ra,rb: glsarray);
    procedure AlignComponents(Former, Latter: TControl; Space: Integer);
  end;

var
  IntegralDialog: TIntegralDialog;

implementation

uses MainForm, Curve_Dialog;

{$R *.xfm}

procedure TIntegralDialog.FormShow(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
begin
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption   := ReadString(S, 'Caption','');
        Label3.Caption           := ReadString(S, 'Label3.Caption','');
        Label1.Caption           := ReadString(S, 'Label1.Caption','');
        Label2.Caption           := ReadString(S, 'Label2.Caption','');
        Panel2.Caption           := ReadString(S, 'Panel2.Caption','');
        CalcButton.Caption       := ReadString(S, 'CalcButton.Caption','');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption','');
        btnCopy.Caption          := ReadString(S, 'btnCopy.Caption','');
        chkPaint.Caption        := ReadString(S, 'chkPaint.Caption','');
      finally
        Free;
      end;
  end;
  if Int_a = -9999 then
    Int_a := Gmin[1];
  if Int_b = -9999 then
    Int_b := Gmax[1];
  edLB.Text := FloatToStr(Int_a);
  edUB.Text := FloatToStr(Int_b);
  chkPaint.Checked := True;
  AlignComponents(Label1, edLB, 5);
  AlignComponents(Label2, edUB, 5);
  AlignComponents(Label3, edResult, 5);
end;

procedure TIntegralDialog.CalcButtonClick(Sender: TObject);
begin
  Int_a := StrToFloat(edLB.Text);
  Int_b := StrToFloat(edUB.Text);
  Calculate;
end;

procedure TIntegralDialog.Calculate;
var
  i, i2, i3, i4: Integer;
  Xi, Yi: glsarray;
begin
  if Spline[Target_of_integral] = True then
    Target_of_integral := SplineDataSeries[Target_of_integral];
  DS := Target_of_integral;
  i2 := 0;
  if chkPaint.Checked then
    Main.LoadDataSeries;
  for i := 1 to NumData[DS] do
    if GY[DS,i] <> 12345678910 then
    begin
      i2 := i2 + 1;
      Xi[i2] := GX[DS,i];
      Yi[i2] := GY[DS,i];
    end;
  if Pos('\', DataSeries[DS]) > 1 then
    SortTwo(i2, Xi, Yi);
  i := 0;
  repeat
    i := i + 1;
  until Xi[i] >= Int_a;
  if i > 1 then
    f_Int_a := (Yi[i] * (Int_a - Xi[i-1]) + Yi[i-1] * (Xi[i] - Int_a)) / (Xi[i] - Xi[i-1])
  else
  begin
    f_Int_a := 0;
    Int_a := Xi[1];
  end;
  i3 := i - 1;
  repeat
    i := i + 1;
  until (Xi[i] >= Int_b) or (i > i2);
  if i <= i2 then
    f_Int_b := (Yi[i] * (Int_b - Xi[i-1]) + Yi[i-1] * (Xi[i] - Int_b)) / (Xi[i] - Xi[i-1])
  else
  begin
    f_Int_b := 0;
    Int_b := Xi[i2];
  end;
  i4 := i;
  TheValue := 0.5 * (f_Int_a + Yi[i3+1]) * (Xi[i3+1] - Int_a);
  for i := i3 + 1 to i4 - 2 do
    TheValue := TheValue + 0.5 * (Yi[i] + Yi[i+1]) * (Xi[i+1] - Xi[i]);
  TheValue := TheValue + 0.5 * (Yi[i4-1] + f_Int_b) * (Int_b - Xi[i4-1]);
  if chkPaint.Checked then
  begin
    for i := 1 to i3 do
    begin
      GX[DS,i] := Xi[i];
      GY[DS,i] := Yi[i];
    end;
    GX[DS,i3+1] := Int_a;
    GY[DS,i3+1] := f_Int_a;
    for i := i3 + 2 to i4 do
    begin
      GX[DS,i] := Xi[i-1];
      GY[DS,i] := Yi[i-1];
    end;
    GX[DS,i4+1] := Int_b;
    GY[DS,i4+1] := f_Int_b;
    for i := i4 + 2 to i2 + 2 do
    begin
      GX[DS,i] := Xi[i-2];
      GY[DS,i] := Yi[i-2];
    end;
    NumData[DS] := i2 + 2;
  end;
  edResult.Text := FloatToStr(CurveDialog.RoundToDecimalSF(TheValue, 9));
end;

procedure TIntegralDialog.SortTwo(n: integer;var ra,rb: glsarray);
(* Programs using routine SORT2 must define type
type
   glsarray = array [1..np] of Double;
in the main routine, with np >= n.   *)
var
  l,j,ir,i: integer;
  rrb,rra: Double;
begin
  l := (n div 2)+1;
  ir := n;
  while true do
  begin
    if (l > 1) then
    begin
      l := l-1;
      rra := ra[l];
      rrb := rb[l]
    end
    else
    begin
      rra := ra[ir];
      rrb := rb[ir];
      ra[ir] := ra[1];
      rb[ir] := rb[1];
      ir := ir-1;
      if (ir = 1) then 
      begin
        ra[1] := rra;
        rb[1] := rrb;
        exit;
      end
    end;
    i := l;
    j := l+l;
    while (j <= ir) do
    begin
      if (j < ir) then
        if (ra[j] < ra[j+1]) then j := j+1;
      if (rra < ra[j]) then
      begin
        ra[i] := ra[j];
        rb[i] := rb[j];
        i := j;
        j := j+j
      end
      else
        j := ir+1
    end;
    ra[i] := rra;
    rb[i] := rrb
  end;
end;
procedure TIntegralDialog.btnCopyClick(Sender: TObject);
begin
  Clipboard.AsText := FloatToStr(TheValue);
  ShowMessage('"' + Clipboard.AsText + '" has been copyed to clipboard.');
end;

procedure TIntegralDialog.HelpButtonClick(Sender: TObject);
{$ifdef Linux}
//var S: String;
{$endif}
begin
  {$ifdef Linux}
  //S := '';
  //QKeyEvent_create(QEventType_KeyPress, Key_F1, 0, 0, @S, False, 0);
  //Main.IndexClick(Sender);
  InvokeHelp;
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
 // Application.HelpCommand(HELP_CONTEXT, 0);
end;

procedure TIntegralDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if chkPaint.Checked = False then
    Target_of_integral := 0;
end;

procedure TIntegralDialog.AlignComponents(Former, Latter: TControl; Space: Integer);
begin
  Latter.Left := Former.Left + Former.Width + Space;
end;
end.
