unit Interpolation;

interface

uses
{$ifdef Linux}
  Math;
{$else}   Math;
{$endif}

type
  RealArray = array[0..1000] of Double;
  TCubicSplineInterpolation = class
  private
    j: Integer;
    p, q, r, s: Double;
  public
    x, y, ydd, yd: RealArray;
    procedure GetSecondDerivatives(n: Integer);
    procedure GetFirstDerivatives(n: Integer);
    function Value(x_: Double): Double;
  //  function Integral(LowerLimit, UpperLimit: Double): Double;
  end;

implementation

procedure TCubicSplineInterpolation.GetSecondDerivatives(n: Integer);
var
  A, B, C, D: RealArray;
  i: Integer;
  procedure SolveTheCoefficientMatrix;
  var
    i: Integer;
    dum: Double;
  begin
    // solving symmetric and tridiagonal matrix using the Thomas algorithm
    for i := 1 to n - 1 do
    begin
      dum := A[i+1] / B[i];
      B[i+1] := B[i+1] - dum * C[i];
      D[i+1] := D[i+1] - dum * D[i];
    end;
    ydd[n] := D[n] / B[n];
    for i := n - 1 downto 1 do
      ydd[i] := (D[i] - C[i] * ydd[i+1]) / B[i];
  end;
begin
  for i := 1 to n - 1 do
  begin
    A[i+1] := (x[i]   - x[i+1]) / 6;
    B[i]   := (x[i-1] - x[i+1]) / 3;
    C[i]   := A[i+1];
    D[i]   := (y[i-1] - y[i]) / (x[i-1] - x[i]) - (y[i] - y[i+1]) / (x[i] - x[i+1]);
  end;
  B[n] := 0;
  D[n] := 0;
  SolveTheCoefficientMatrix;
  j := -1;
end;

function TCubicSplineInterpolation.Value(x_: Double): Double;
var k: Integer;
begin
  k := 0;
  repeat
    Inc(k);
  until x[k] >= x_;
  if j <> k then
  begin
    p := (ydd[k-1] - ydd[k]) / (x[k-1] - x[k]);
    q := (x[k-1] * ydd[k] - x[k] * ydd[k-1]) / (x[k-1] - x[k]);
    r := (y[k-1] - y[k]) / (x[k-1] - x[k]) - p * (x[k-1] * x[k-1] + x[k-1] * x[k] + x[k] * x[k]) / 6 - 0.5 * q * (x[k-1] + x[k]);
    s := y[k] - p * Power(x[k], 3.0) / 6 - 0.5 * q * x[k] * x[k] - r * x[k];
  end;
  j := k;
  Value := p * Power(x_, 3.0) / 6 + 0.5 * q * x_ * x_ + r * x_ + s;
end;

procedure TCubicSplineInterpolation.GetFirstDerivatives(n: Integer);
var k: Integer;
begin
  for k := 1 to n do
  begin
    p := (ydd[k-1] - ydd[k]) / (x[k-1] - x[k]);
    q := (x[k-1] * ydd[k] - x[k] * ydd[k-1]) / (x[k-1] - x[k]);
    r := (y[k-1] - y[k]) / (x[k-1] - x[k]) - p * (x[k-1] * x[k-1] + x[k-1] * x[k] + x[k] * x[k]) / 6 - 0.5 * q * (x[k-1] + x[k]);
    s := y[k] - p * Power(x[k], 3.0) / 6 - 0.5 * q * x[k] * x[k] - r * x[k];
    yd[k-1] := 0.5 * p * x[k-1] * x[k-1] + q * x[k-1] + r;
  end;
end;

end.
