unit Legend_Dialog;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls, QComCtrls, QImgList, QMenus, 
  Title_Editor, IniFiles;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ToolWin, ImgList, Menus, Title_Editor, IniFiles;
{$endif}

type
  TLegendDialog = class(TForm)
    Panel1: TPanel;
    OK: TButton;
    CancelButton: TButton;
    Label7: TLabel;
    edLineSpacing: TEdit;
    Label8: TLabel;
    edRshift: TEdit;
    Label9: TLabel;
    edDshift: TEdit;
    Panel2: TPanel;
    Label1: TLabel;
    edMargin: TEdit;
    cbBorderStyle: TComboBox;
    Label2: TLabel;
    TitleEditor: TTitleEditor;
    HelpButton: TButton;
    procedure OKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbBorderStyleChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TitleEditorEnter(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure BoldClick(Sender: TObject);
    procedure TitleEditorExit(Sender: TObject);
  private

  public

  end;

var
  LegendDialog: TLegendDialog;
  Msg : array[1..6] of String;

implementation

uses MainForm, Help_Search;

{$R *.xfm}

//----------------------------------------------------------

procedure TLegendDialog.FormCreate(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
  procedure GetComboBoxItems(CB: TComboBox );
  begin
    i := CB.ItemIndex;
    CB.Items.Text := StringReplace(IniFile.ReadString(S, CB.Name +  '.Items.Text', ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
    CB.ItemIndex := i;
  end;
begin
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  Font.Assign(Main.Font);
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        Caption                  := ReadString(S, 'Caption', '');
        Label7.Caption           := ReadString(S, 'Label7.Caption', '');
        Label8.Caption           := ReadString(S, 'Label8.Caption', '');
        Label9.Caption           := ReadString(S, 'Label9.Caption', '');
        CancelButton.Caption     := ReadString(S, 'CancelButton.Caption', '');
        HelpButton.Caption       := ReadString(S, 'HelpButton.Caption', '');
        Label1.Caption           := ReadString(S, 'Label1.Caption', '');
        Label2.Caption           := ReadString(S, 'Label2.Caption', '');
        GetComboBoxItems(cbBorderStyle);
        for i := 1 to 2 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  if LANG = 128 then
  begin
    TitleEditor.FontComboBox.Items.Add(Msg[1]);
    TitleEditor.FontComboBox.Items.Add(Msg[2]);
    Label1.Left := 238;
    Label7.Left := 60;
    Label8.Left := 60;
    Label9.Left := 60;
  end
end;

procedure TLegendDialog.FormShow(Sender: TObject);
begin
  dumstr := Legend[target];
  TitleEditor.Memo.Lines.Clear;
  while dumstr <> '' do
  begin
    TitleEditor.Memo.Lines.Add(Copy(dumstr, 1, Pos('\n', dumstr) - 1));
    dumstr := Copy(dumstr, Pos('\n', dumstr) + 2, 500);
  end;
  edLineSpacing.Text := FormatFloat('#0.0', LegendLspacing[target]);
  edRshift.Text := FormatFloat('##0.0', LegendXshift[target]);
  edDshift.Text := FormatFloat('##0.0', LegendYshift[target]);
  cbBorderStyle.ItemIndex := lblBorderType[target];
  edMargin.Text := FloatToStr(lblMargin[target]);
end;
//----------------------------------------------------------
procedure TLegendDialog.OKClick(Sender: TObject);
var i : Integer;
begin
  Legend[target] := '';
  with TitleEditor.Memo do
    for i := 0 to Lines.Count - 1 do
      if (i <> Lines.Count - 1) or (Lines[i] <> '') then
        Legend[target] := Legend[target] + Lines[i] + '\n';
  LegendLspacing[target]:= StrToFloat(edLineSpacing.Text);
  LegendXshift[target]  := StrToFloat(edRshift.Text);
  LegendYshift[target]  := StrToFloat(edDshift.Text);
  lblBorderType[target]  := cbBorderStyle.ItemIndex;
  lblMargin[target] := StrToFloat(edMargin.Text);
  AllowWrite := True;
  Main.WriteLegend(Main.GraphImage.Canvas, asc, target, target);
  Modified[NumFigure] := 1;
end;

procedure TLegendDialog.cbBorderStyleChange(Sender: TObject);
begin
  if cbBorderStyle.ItemIndex = 0 then
    edMargin.Enabled := False
  else
    edMargin.Enabled := True;
end;

procedure TLegendDialog.TitleEditorEnter(Sender: TObject);
begin
  TitleEditor.DefaultFont := Main.ToolBar.Font;// .GraphImage.Canvas.Font;
  TitleEditor.DefaultFont.Color := clBlack;
  TitleEditor.HyperText.Shadow := False;
 // TitleEditor.ToolBar.Font := Main.GraphImage.Canvas.Font;
  PopupMenu := TitleEditor.PopupMenu;
  TitleEditor.FrameEnter(Sender);
end;

procedure TLegendDialog.HelpButtonClick(Sender: TObject);
begin
  {$ifdef Linux}
  //Main.IndexClick(Sender);
  InvokeHelp;
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
end;

procedure TLegendDialog.BoldClick(Sender: TObject);
begin
  TitleEditor.tbBoldClick(Sender);
end;

procedure TLegendDialog.TitleEditorExit(Sender: TObject);
begin
  PopupMenu := nil;
end;

end.
