unit NonLinearFitting;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, Math, MathsParser, Curve_Dialog;
{$else}  
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Math, MathsParser, Curve_Dialog;
{$endif}

type
  glndata  = array [1..2000] of Double;
  glma     = array [1..8] of Double;
  glmabyma = array [1..8,1..8] of Double;
  TMarquadt = class(TComponent)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  private
    Parser : TMathsParser;
    procedure EstimatedValues(a: glma; x: glndata);
    procedure mrqcof(x, y, sig: glndata; var a, ia: glma; var alpha_: glmabyma; var beta_: array of Double; var ObjFunc: Double; ndata: Integer);
    procedure gaussj(var a, b: glmabyma; n, m: Integer);
  public
    IsValid: Boolean;
    procedure Optimize(var a: glma);
    procedure mrqmin(x, y, sig: glndata; var a, ia: glma; var covar, alpha_: glmabyma; var ObjFunc, fgauss, alamda: Double; ndata: Integer);
    function RMSE(i: Integer; a: glma): Double;
  end;

var
  Marquadt: TMarquadt;
  NumPerform, ma, Counter: Integer;
  beta_: array[0..20] of Double;
  ndata: Integer;
  ymod, ymod0: glndata;
  Yavg: Double;

implementation

uses MainForm;

///////////////////   Optimization argorithsm of Marquardt  /////////////////////////
constructor TMarquadt.Create(AOwner: TComponent);
var Equation : String;
begin
  inherited Create(AOwner);
  Parser := TMathsParser.Create(Self);
  Equation := StringReplace(UsersEquation, '#', Settler, [rfReplaceAll]);
  Parser.DetectVariables(Equation, 'x, y');
  Parser.V8Name := 'x';
  {
  Equation := StringReplace(Equation, 'a1', 'A', [rfReplaceAll]);
  Equation := StringReplace(Equation, 'a2', 'B', [rfReplaceAll]);
  Equation := StringReplace(Equation, 'a3', 'C', [rfReplaceAll]);
  Equation := StringReplace(Equation, 'a4', 'D', [rfReplaceAll]);
  Equation := StringReplace(Equation, 'a5', 'E', [rfReplaceAll]);
  Equation := StringReplace(Equation, 'a6', 'F', [rfReplaceAll]);
  Equation := StringReplace(Equation, 'a7', 'G', [rfReplaceAll]);
  }
  try
    Parser.Expression := Equation;
    IsValid := True;
  except
    {$ifdef Linux}
    if Application.MessageBox(PChar(CurveDialog.Msg[21]),
                              PChar(Parser.ErrMsg), [smbYes, smbNo]) = smbYes then
    {$else}
    if Application.MessageBox(PChar(CurveDialog.Msg[21]),
                              PChar(Parser.ErrMsg), MB_YESNO + MB_ICONWARNING) = IDYES then
    {$endif}
      Parser.ShowHelp;
    IsValid := False;
  end;
end;

procedure TMarquadt.Optimize(var a: glma);
var
  i, i2, itst, j: Integer;
  alamda, ObjFunc, OFbase, fgauss: Double;
  x, y, sig: glndata;
  covar, alpha_: glmabyma;
  ia: glma;
begin
  numRMSEcall := 0;
  ma := 7;
  i := DS;
  for j := 1 to ma do
  begin
    if not Pfix[i,j] then
      ia[j] := 1
    else
      ia[j] := 0;
    a[j] := P[i,j];
  end;
  i := ObjDataSeries[i];
  i2 := 0;
  Yavg := 0;
  for j := 1 to NumData[i] do
    if (GX[i,j] >= CurveDialog.C_Xmin) and (GX[i,j] <= CurveDialog.C_Xmax) and (GX[i,j] <> 12345678910) then
    begin
      i2 := i2 + 1;
      x[i2] := GX[i,j];   // Not used here.
      y[i2] := GY[i,j];
      Yavg := Yavg + y[i2];
    end;
  Yavg := Yavg / i2;
  //for j := 1 to NumData[i] do
  //  sig[j] := dum / i2 * 0.1;
  ndata := i2;
  alamda := -1;
  ObjFunc := 0; itst := 0; fgauss := 0;
//-------------------------------------------------------------------------
  mrqmin(x, y, sig, a, ia, covar, alpha_, ObjFunc, fgauss, alamda, ndata);
//-------------------------------------------------------------------------
  while itst < 7 do //) and (alamda > 0.00001) do
  begin
    OFbase := ObjFunc;
  //------------------------------------------------------------------------
    mrqmin(x, y, sig, a, ia, covar, alpha_, ObjFunc, fgauss, alamda, ndata);
  //------------------------------------------------------------------------
    if ObjFunc > OFbase then
      itst := 0
    else
    begin
      if Abs(OFbase - ObjFunc) < 0.1 Then itst := itst + 1;
    // if (Abs(OFbase - ObjFunc) > 0.0000000000000001) and (Abs(OFbase - ObjFunc) <= 0.0000001) then break;
      if numRMSEcall > Precision[DS] then
        Exit;
    end;
  end;
 // if alamda < 0.00001 then
  begin
    alamda := 0;
  //------------------------------------------------------------------------
    mrqmin(x, y, sig, a, ia, covar, alpha_, ObjFunc, fgauss, alamda, ndata);
  //------------------------------------------------------------------------
  end;
end;

//====================================================================================
procedure TMarquadt.mrqmin(x, y, sig: glndata; var a, ia: glma; var covar, alpha_: glmabyma; var ObjFunc, fgauss, alamda: Double; ndata: Integer);
//====================================================================================
var
  j, k, l, mfit: Integer;
  OFbase: Double;
  da: array[0..100] of Double;
  atry: glma;
  dumarray: glmabyma;
  //i: Integer;
  //S: String;
begin
  mfit := 0;
  for j := 1 to ma do
    if ia[j] <> 0 then mfit := mfit + 1;
  if alamda < 0 Then
  begin
    alamda := 0.01;
  //------------------------------------------------------------------
    mrqcof(x, y, sig, a, ia, alpha_, beta_, ObjFunc, ndata);
  //------------------------------------------------------------------
    OFbase := ObjFunc;
    for j := 1 to ma do atry[j] := a[j];
  end
  else
    OFbase := ObjFunc; //? ɤɬ
  for j := 1 to mfit do
  begin
    for k := 1 to mfit do covar[j,k] := alpha_[j,k];
    covar[j,j] := alpha_[j,j] * (1 + alamda);
    da[j] := beta_[j];
    dumarray[j,1] := da[j]
  end;
//----------------------------------------------------------
  gaussj(covar, dumarray, mfit, 1);
//----------------------------------------------------------
  for j := 1 to mfit do da[j] := dumarray[j,1];
  if alamda = 0 then
  begin
    {
    dumstr := '';
    for j := 1 to mfit do
      dumstr := dumstr + '  ' + FloatToStr(Sqrt(Abs(covar[j,j]))) + #13 + #10;
    ShowMessage(dumstr);
    dumstr := '';
    for i := 1 to mfit do
    begin
      for j := 1 to mfit do
        if i >= j then
        begin
          if covar[i,j] < 0 then S := '' else S := ' ';
          dumstr := dumstr + ' ' + S + FormatFloat('0.00', covar[i,j]/(Sqrt(Abs(covar[i,i])) * Sqrt(Abs(covar[j,j]))));
        end;
      dumstr := dumstr + #13 + #10;
    end;
    ShowMessage(dumstr);
    // }
    Exit;
  end;
  j := 0;
  for l := 1 to ma do
    if ia[l] <> 0 Then
    begin
      j := j + 1;
   //   if a[l] + da[j] > a_max[l] then da[j] := 0;   // 990524
   //   if a[l] + da[j] < a_min[l] then da[j] := 0;   // 990524
      atry[l] := a[l] + da[j];
   end;
  //  else
  //    atry[l] := a[l]; //? ɤɬ
//------------------------------------------------------------------
  mrqcof(x, y, sig, atry, ia, covar, da, ObjFunc, ndata);
//------------------------------------------------------------------
  if ObjFunc < OFbase then
  begin
    alamda := 0.1 * alamda;
    //OFbase :=ObjFunc;
    for j := 1 to mfit do
      begin
      for k := 1 to mfit do alpha_[j,k] := covar[j,k];
      beta_[j] := da[j];
      end;
    for l := 1 to ma do a[l] := atry[l];
  end
  else
  begin
    alamda := 10 * alamda;
    ObjFunc := OFbase
  end;
end;

//====================================================================================
procedure TMarquadt.gaussj(var a, b: glmabyma; n, m: Integer);
//====================================================================================
var
  i, icol, irow, j, k, l, ll:Integer;
  big, dum, pivinv: Double;
  i2 :Integer;
  indxr, indxc: array[-1..8] of Integer;
  //:array[1..5] of integer;
  ipiv: array[1..8] of Integer;
const NMAX = 50;
begin
  icol := 0; irow := 0;
  for i := 1 to n do
    begin
    indxc[i]:=0;
    indxr[i]:=0;
    ipiv[i]:=0;
    end;
  for j := 1 to n do ipiv[j]:=0;
  for i := 1 to n do
    begin
    big := 0;
    for j := 1 to n do
      begin
      if ipiv[j] <> 1 then
        begin
        for k := 1 to n do
          begin
          if ipiv[k] = 0 then
            begin
            if abs(a[j][k]) >= big then
              begin
              big := abs(a[j][k]);
              irow := j;
              icol := k;
              end;
            end
          else if ipiv[k] > 1 then begin ShowMessage('singular matrix in gaussj'); exit; end;
          end;
        end;
      end;
    ipiv[icol]:=ipiv[icol] + 1;
    //==========
    if irow <> icol then
      begin
      for l := 1 to n do
        begin
        dum := a[irow][l];
        a[irow][l] := a[icol][l];
        a[icol][l] := dum;
        end;
      for l := 1 to m do
        begin
        dum := b[irow][l];
        b[irow][l] := b[icol][l];
        b[icol][l] := dum;
        end;
      end;
    indxr[i] := irow;
    indxc[i] := icol;
    if a[icol][icol]= 0 then begin ShowMessage('singular matrix in gaussj'); exit;end;
    pivinv := 1 / a[icol][icol];
    a[icol][icol] := 1;
    for l := 1 to n do a[icol][l] := a[icol][l]*pivinv;
    for l := 1 to m do b[icol][l] := b[icol][l]*pivinv;
    for ll := 1 to n do
      begin
      if ll <> icol then
        begin
        dum := a[ll][icol];
        a[ll][icol] := 0;
        for l := 1 to n do a[ll][l]:=a[ll][l]- a[icol][l]* dum;
        for l := 1 to m do b[ll][l]:= b[ll][l]-b[icol][l]* dum;
        end;
      end;
  end;
  for l := n downto 1 do
    begin
    if indxr[l] <> indxc[l] then
      begin
      for k := 1 to n do
        begin
        dum := a[k][indxr[l]];
        a[k][indxr[l]] := a[k][indxc[l]];
        a[k][indxc[l]] := dum;
        end;
      end;
    end;
end;

//====================================================================================
procedure TMarquadt.mrqcof(x, y, sig: glndata; var a, ia: glma; var alpha_: glmabyma; var beta_: array of Double; var ObjFunc: Double; ndata: Integer);
//====================================================================================
//const
  //da = 0.0000000005; { <-- This value should be chosen carefully.}
  //MMAX = 8;
var
  mfit, i, j, k, l, m: Integer;
  da, dy, sig2i, wt: Double;
  dyda, ymod1: array[0..2000, 0..8] of Double;
  dumintarray: array[1..10] of integer;
begin
  mfit := 0;
  for i := 1 to 1000 do
  begin
    ymod[i] := 0;
    ymod0[i] := 0;
  end;
  for j := 1 to ma do
    if ia[j] <> 0 then mfit := mfit + 1;
  for j := 1 to mfit do
  begin
    for k := 1 to j do alpha_[j,k] := 0;
    beta_[j] := 0;
  end;
  ObjFunc := 0;
//----------------------------------------------------------
  EstimatedValues(a, x);
//----------------------------------------------------------
  {
  if Yl[ObjDataSeries[DS]] = 1 then
    j := 3
  else
    j := 4;
  if logarithm[j] = 1 then
    for i := 1 to ndata do
      ymod0[i] := ln(ymod[i])
  else
  }
  for i := 1 to ndata do
    ymod0[i] := ymod[i];
  for k := 1 to ma do
  begin
    if ia[k] <> 0 then
    begin
      if a[k] <> 0 then
        da := a[k] * 0.000001
      else
        da := 0.00001;
      a[k] := a[k] + da;
    //----------------------------------------------------------
      EstimatedValues(a, x);
    //----------------------------------------------------------
      {
      if Yl[ObjDataSeries[DS]] = 1 then
        j := 3
      else
        j := 4;
      }
      for i := 1 to ndata do
      begin
        if x[i] = 12345678910 then Continue;
        {
        if logarithm[j] = 1 then
          ymod1[i,k] := ln(ymod[i])
        else
        }
        ymod1[i,k] := ymod[i];
        dyda[i,k]  := (ymod1[i,k] - ymod0[i]) / da;
        sig[i] := Abs(ymod1[i,k] - ymod0[i]) + Yavg * 0.001;
      end;
      a[k] := a[k] - da;
    end;
  end;
  for i := 1 to ndata do
  begin
    if x[i] = 12345678910 then Continue;
    sig2i := 1 / (sig[i] * sig[i]);
    dy := y[i] - ymod0[i];
    j  := 0;
    for l := 1 to ma do
    begin
      if ia[l] <> 0 then
      begin
        j := j + 1;
        wt := dyda[i,l] * sig2i;
        k := 0;
        for m := 1 to l do
          if ia[m] <> 0 then
          begin
            k := k + 1;
            alpha_[j,k] := alpha_[j,k] + wt * dyda[i,m];
          end;
        beta_[j] := beta_[j] + dy * wt;
      end;
    end;
  {============ Objective Function is here! ===========}
    ObjFunc :=ObjFunc + dy * dy; // * sig2i;
  {====================================================}
  end;
  for j := 2 to mfit do
    for k := 1 to j - 1 do alpha_[k,j] := alpha_[j,k];
end;

//----------------------------------------------------------
procedure TMarquadt.EstimatedValues(a: glma; x: glndata);
var
  i : Integer;
 // Equation: String;
begin
  numRMSEcall := numRMSEcall + 1;
  //Counter := 0;
  Parser.V1 := a[1];
  Parser.V2 := a[2];
  Parser.V3 := a[3];
  Parser.V4 := a[4];
  Parser.V5 := a[5];
  Parser.V6 := a[6];
  Parser.V7 := a[7];
  // Equation := AnsiLowerCase(Equation);
  Counter := 0;
  for i := 1 to ndata do
  begin
  //  dumstr := StringReplace(Equation, 'x', FloatToStr(x[i]), [rfReplaceAll]);
  //  dumstr := StringReplace(dumstr, 'e' + FloatToStr(x[i]) + 'p', 'exp', [rfReplaceAll]);
    Parser.V8 := x[i];
    ymod[i] := Parser.Value;// CurveDialog.UserDefinedFunction(dumstr);
    Counter := Counter + 1;
  end;
end;

//----------------------------------------------------------
function TMarquadt.RMSE(i: Integer; a: glma): Double;
var
  ycal: Double;
  i2, j, Counter: Integer;
begin
  j := ObjDataSeries[i];
  numRMSEcall := numRMSEcall + 1;
  RMSE := 0;
  Counter := 0;
  //dumint := Pos('[', DataSeries[i]);
  //Equation := StringReplace(UsersEquation, '#', Settler, [rfReplaceAll]);
  //for i2 := 1 to 8 do
  //  Equation := StringReplace(Equation, 'a' + IntToStr(i2), FloatToStr(a[i2]), [rfReplaceAll]);
  for i2 := 1 to NumData[j] do
    if (GX[j,i2] >= CurveDialog.C_Xmin) and (GX[j,i2] <= CurveDialog.C_Xmax) and (GX[j,i2] <> 12345678910) then
    begin
      //dumstr := StringReplace(Equation, 'x', FloatToStr(GX[j,i2]), [rfReplaceAll]);
      //dumstr := StringReplace(dumstr, 'e' + FloatToStr(GX[j,i2]) + 'p', 'exp', [rfReplaceAll]);
      Parser.V8 := GX[j,i2];
      ycal := Parser.Value;//   CurveDialog.UserDefinedFunction(dumstr);
      Result := Result + Sqr(GY[j,i2] - ycal);
      Counter := Counter + 1;
    end;
  Result := Sqrt(Result / Counter);
end;

destructor TMarquadt.Destroy;
begin
  Parser.Free;
  inherited Destroy;
end;

end.
