unit Title_Editor;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QMenus, QImgList, QComCtrls, QStdCtrls, HyperText, 
  QExtCtrls, CLCBox, QTypes, Qt, IniFiles
  ;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ImgList, ComCtrls, StdCtrls, ToolWin, HyperText, ExtCtrls, CLCBox,
  Registry, IniFiles;
{$endif}

type
  TTitleEditor = class(TFrame)
    Memo: TMemo;
    ToolBar1: TToolBar;
    FontComboBox: TComboBox;
    SizeComboBox: TComboBox;
    tbBold: TToolButton;
    tbItalic: TToolButton;
    tbSuper: TToolButton;
    tbSub: TToolButton;
    tbUndo: TToolButton;
    ImageList1: TImageList;
    DummyImage: TImage;
    tbShadow: TToolButton;
    HyperText: THyperText;
    ColorLookupComboBox: TColorLookupComboBox;
    PopupMenu: TPopupMenu;
    pmUndo: TMenuItem;
    N1: TMenuItem;
    pmCut: TMenuItem;
    pmCopy: TMenuItem;
    pmPaste: TMenuItem;
    N2: TMenuItem;
    pmBold: TMenuItem;
    pmItalic: TMenuItem;
    pmSup: TMenuItem;
    pmSub: TMenuItem;
    pmSelectAll: TMenuItem;
    procedure SizeComboBoxChange(Sender: TObject);
    procedure FrameEnter(Sender: TObject);
    procedure FontComboBoxChange(Sender: TObject);
    procedure tbBoldClick(Sender: TObject);
    procedure tbItalicClick(Sender: TObject);
    procedure tbSuperClick(Sender: TObject);
    procedure tbSubClick(Sender: TObject);
    procedure tbUndoClick(Sender: TObject);
    procedure MemoKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MemoMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ColorLookupComboBoxChange(Sender: TObject);
    procedure tbShadowClick(Sender: TObject);
    procedure pmUndoClick(Sender: TObject);
    procedure pmCutClick(Sender: TObject);
    procedure pmCopyClick(Sender: TObject);
    procedure pmPasteClick(Sender: TObject);
    procedure pmSelectAllClick(Sender: TObject);
  private
    {$ifdef Linux}
    str: array[0..9] of WideString;
    {$else}
    str: array[0..9] of String;
    {$endif}
    Msg: array[1..4] of String;
    asc: Single;
    procedure PrepareInsertion;
    {$ifdef Linux}
    procedure InsertTag(S1, S2: WideString);
    {$else}
    procedure InsertTag(S1, S2: String);
    {$endif}
    procedure GetCurrentFont;
  public
    DefaultFont: TFont;
    mags: Double;
    dum: Double;
  end;

implementation

{$R *.xfm}

var AllowInsertion: Boolean;

procedure TTitleEditor.PrepareInsertion;
var
  {$ifdef Linux}
  S: WideString;
  {$else}
  S: String;
  {$endif}
  i, j: Integer;
begin
  j := Memo.SelStart;
  with Memo do
    if j > 0 then
    begin
      S := Copy(Text, j, 1);
      if Pos('[', S) <> 0 then
        Memo.SelStart := j + 2
      else
      begin
        S := Copy(Text, j - 1, 1);
        if Pos('[', S) <> 0 then
          Memo.SelStart := j + 1;
      end;
      S := Copy(Text, Memo.SelStart + Memo.SelLength, 1);
      if Pos('[', S) <> 0 then
        Memo.SelStart := Memo.SelLength - 1
      else
      begin
        S := Copy(Text, Memo.SelStart + Memo.SelLength - 1, 1);
        if Pos('[', S) <> 0 then
          Memo.SelStart := Memo.SelLength - 2;
      end;
      dum := 12321;
      for i := 0 to 9 do
        str[i] := Lines[i];
    end;
end;

procedure TTitleEditor.SizeComboBoxChange(Sender: TObject);
var
  i, j: Integer;
  {$ifdef Linux}
  S1, S2 : WideString;
  {$else}
  S1, S2 : String;
  {$endif}
begin
  if AllowInsertion then
  begin
    PrepareInsertion;
    if DefaultFont.Size < 14 then
      S2 := '[' + IntToStr(DefaultFont.Size - 7) + ']'
    else
    begin
      if DefaultFont.Size = 14 then S2 := '[6]';
      if DefaultFont.Size = 16 then S2 := '[7]';
      if DefaultFont.Size = 20 then S2 := '[8]';
      if DefaultFont.Size >=24 then S2 := '[9]';
    end;
    with Memo do
    begin
      S1 := SelText;
      for j := 0 to 9 do
      begin
        i := Pos('[' + IntToStr(j) + ']', SelText);
        if i > 0 then
          Delete(S1, i, 3);
      end;
      i := Memo.SelLength;
      Seltext := '[' + IntToStr(SizeComboBox.ItemIndex) + ']' + S1;
      if i > 0 then
        SelText := SelText + S2;
    end;
  end;
end;

procedure TTitleEditor.FrameEnter(Sender: TObject);
var
  IniFile: TIniFile;
  S, ResourceFileName: String;
  i: Integer;
begin
  S := Self.Name;
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        pmUndo.Caption           := ReadString(S, 'pmUndo.Caption','');
        pmCut.Caption            := ReadString(S, 'pmCut.Caption','');
        pmCopy.Caption           := ReadString(S, 'pmCopy.Caption','');
        pmPaste.Caption          := ReadString(S, 'pmPaste.Caption','');
        pmSelectAll.Caption      := ReadString(S, 'pmSelectAll.Caption','');
        pmBold.Caption           := ReadString(S, 'pmBold.Caption','');
        pmItalic.Caption         := ReadString(S, 'pmItalic.Caption','');
        pmSup.Caption            := ReadString(S, 'pmSup.Caption','');
        pmSub.Caption            := ReadString(S, 'pmSub.Caption','');
        tbShadow.Hint            := ReadString(S, 'tbShadow.Hint','');
        for i := 1 to 4 do
          Msg[i] := StringReplace(ReadString(S, 'Msg' + IntToStr(i), ''), '<br>',   #13#10 , [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
  tbBold.Hint   := pmBold.Caption;
  tbItalic.Hint := pmItalic.Caption;
  tbSuper.Hint  := pmSup.Caption;
  tbSub.Hint    := pmSub.Caption;
  tbUndo.Hint   := pmUndo.Caption;
  asc := Screen.Height / (0.6 * 25.4 * 14);
  mags := 1;
  pmSup.ShortCut := $4000 + $26; //See Win32 Developer's Referrences with 'Virtual Key Codes'
  pmSub.ShortCut := $4000 + $28;
  FontComboBox.Items.Clear;
  FontComboBox.Items.Add(HyperText.FontNameA);
  FontComboBox.Items.Add(HyperText.FontNameS);
  FontComboBox.Items.Add(HyperText.FontNameR);
  FontComboBox.Items.Add(HyperText.FontNameV);
  FontComboBox.Items.Add(HyperText.FontNameG);
  FontComboBox.Items.Add(HyperText.FontNameM);
end;

procedure TTitleEditor.FontComboBoxChange(Sender: TObject);
var
  {$ifdef Linux}
  S1, S2 : WideString;
  {$else}
  S1, S2 : String;
  {$endif}
  i: Integer;
begin
  if AllowInsertion then
  begin
    PrepareInsertion;
    S2 := '[a]';
    {$ifdef Linux}
    if DummyImage.Canvas.Font.Name = 'kochi mincho' then S2 := '[m]';
    if DummyImage.Canvas.Font.Name = 'kochi gothic' then S2 := '[g]';
    if DummyImage.Canvas.Font.Name = 'symbol'       then S2 := '[s]';
    if DummyImage.Canvas.Font.Name = 'times'        then S2 := '[r]';
    if DummyImage.Canvas.Font.Name = 'lucida'       then S2 := '[v]';
    {$else}
    if DummyImage.Canvas.Font.Name = Msg[1]            then S2 := '[m]';
    if DummyImage.Canvas.Font.Name = Msg[2]            then S2 := '[g]';
    if DummyImage.Canvas.Font.Name = 'Symbol'          then S2 := '[s]';
    if DummyImage.Canvas.Font.Name = 'Times New Roman' then S2 := '[r]';
    if DummyImage.Canvas.Font.Name = 'Verdana'         then S2 := '[v]';
    {$endif}
    with Memo do
    begin
      S1 := SelText;
      i := Pos('[a]', SelText); if i <> 0 then Delete(S1, i, 3);
      i := Pos('[g]', SelText); if i <> 0 then Delete(S1, i, 3);
      i := Pos('[m]', SelText); if i <> 0 then Delete(S1, i, 3);
      i := Pos('[s]', SelText); if i <> 0 then Delete(S1, i, 3);
      i := Pos('[r]', SelText); if i <> 0 then Delete(S1, i, 3);
      i := Pos('[v]', SelText); if i <> 0 then Delete(S1, i, 3);
      i := Memo.SelLength;
      case FontComboBox.ItemIndex of
      0: SelText := '[a]' + S1;
      1: SelText := '[s]' + S1;
      2: SelText := '[r]' + S1;
      3: SelText := '[v]' + S1;
      4: SelText := '[g]' + S1;
      5: SelText := '[m]' + S1;
      end;
      if i > 0 then SelText := SelText + S2;
    end;
  end;
end;

{$ifdef Linux}
procedure TTitleEditor.InsertTag(S1, S2: WideString);
{$else}
procedure TTitleEditor.InsertTag(S1, S2: String);
{$endif}
var
  i: Integer;
  {$ifdef Linux}
  S: WideString;
  {$else}
  S: String;
  {$endif}
begin
  if AllowInsertion then
  begin
    PrepareInsertion;
    with Memo do
    begin
      if Memo.SelLength > 0 then
      begin
        i := Pos('[' + S1 + ']', SelText);
        S := SelText;
        if i <> 0 then
          Delete(S, i, 3);
        i := Pos('[' + S2 + ']', S);
        if i <> 0 then
          Delete(S, i, 3);
        SelText := '[' + S1 + ']' + S + '[' + S2 + ']'
      end
      else
        SelText := '[' + S1 + ']';
      Memo.SelStart := Memo.SelStart + Memo.SelLength;
    end;
  end;
end;

procedure TTitleEditor.tbBoldClick(Sender: TObject);
begin
  InsertTag('b', '/b');
end;

procedure TTitleEditor.tbItalicClick(Sender: TObject);
begin
  InsertTag('i', '/i');
end;

procedure TTitleEditor.tbShadowClick(Sender: TObject);
begin
  InsertTag('#', '/#');
end;

procedure TTitleEditor.tbSuperClick(Sender: TObject);
begin
  InsertTag('^', '-');
end;

procedure TTitleEditor.tbSubClick(Sender: TObject);
begin
  InsertTag('_', '-');
end;

procedure TTitleEditor.tbUndoClick(Sender: TObject);
var i: Integer;
begin
  if dum = 12321 then
  begin
    Memo.Lines.Clear;
    for i := 0 to 9 do
      if str[i] <> '' then
        Memo.Lines.Add(str[i])
  end
  else
    Memo.Undo;// SendMessage(Memo.Handle, WM_UNDO, 0, 0);
end;

procedure TTitleEditor.MemoKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {
  if (Shift = [ssCtrl]) then
  begin
    if (Key = Word('i')) or (Key = Word('I')) then
      tbItalicClick(Sender);
    if (Key = Word('b')) or (Key = Word('B')) then
      tbBoldClick(Sender);
    if (Key = VK_UP) then
      tbSuperClick(Sender);
    if (Key = VK_DOWN) then
      tbSubClick(Sender);
  end;
  }
  GetCurrentFont;
end;

procedure TTitleEditor.MemoMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  GetCurrentFont;
end;

procedure TTitleEditor.GetCurrentFont;
var
  {$ifdef Linux}
  S: WideString;
  {$else}
  S: String;
  {$endif}
  row, i, i2, j, x, y, direction: Integer;
  function GetCurCol: Integer;
  {$ifdef Linux}
  var
    j: Integer;
    pi, pj: PInteger;
    {$ifdef Linux}
    S: WideString;
    {$else}
    S: String;
    {$endif}
  begin
    New(pi);
    New(pj);
    QMultiLineEdit_getCursorPosition(Memo.Handle, pi, pj);
    j := pj^;
    dispose(pi);
    dispose(pj);
    S := Copy(Memo.Text, Memo.SelStart - j + 1, j);
    j := Length(S);
    Result := j;
  {$else}
  begin
    Result := 1 + Memo.SelStart - SendMessage(Memo.Handle, EM_LINEINDEX, row, 0);
  {$endif}
  end;
  function GetCurLine: Integer;
  {$ifdef Linux}
  var
    pi, pj: PInteger;
  begin
    New(pi);
    New(pj);
    QMultiLineEdit_getCursorPosition(Memo.Handle, pi, pj);
    Result := pi^;
    dispose(pi);
    dispose(pj);
  {$else}
  begin
    Result := SendMessage(Memo.Handle, EM_LINEFROMCHAR, Memo.SelStart, 0);
  {$endif}
  end;
begin
  AllowInsertion := False;
  row := GetCurLine;
  {
  IniSelStart  := Memo.SelStart;
  IniSelLength := Memo.SelLength;
  IniSelText   := Memo.SelText;
  }
  (*
  {$ifdef Linux}
  if Memo.SelLength > 1 then
  {$else}
  if row <> SendMessage(Memo.Handle, EM_LINEFROMCHAR, Memo.SelStart + Memo.SelLength, 0) then
  {$endif}
  begin
    FontComboBox.Enabled := False;
    SizeComboBox.Enabled := False;
    tbBold.Enabled    := False;
    tbItalic.Enabled  := False;
    tbSuper.Enabled   := False;
    tbSub.Enabled     := False;
   // Memo.SelLength := 0;
    Exit;
  end
  else
  begin
    FontComboBox.Enabled := True;
    SizeComboBox.Enabled := True;
    tbBold.Enabled    := True;
    tbItalic.Enabled  := True;
    tbSuper.Enabled   := True;
    tbSub.Enabled     := True;
  end;
  *)
  i := GetCurCol;
  DummyImage.Canvas.Font := DefaultFont;
  HyperText.TargetCanvas := DummyImage.Canvas;
  //Main.SetDefaultFont(Canvas, asc);
  x := - 100;
  y := - 100;
  direction := 0;
  if row > 0 then                           // 2/10
    for i2 := 0 to row - 1 do
    begin
      S := Memo.Lines[i2];
      S := Copy(S, 0, 1000);
      HyperText.Write(S, x, y, direction);
    end;
  S := Memo.Lines[row];
  S := Copy(S, 0, i);
  HyperText.Write(S, x, y, direction);
  with DummyImage.Canvas.Font do
  begin
    if (Style = [fsBold]) or (Style = [fsBold, fsItalic]) then
    begin
      tbBold.Down    := True;
      tbBold.Enabled := False;
    end
    else
    begin
      tbBold.Down    := False;
      tbBold.Enabled := True;
    end;
    if (Style = [fsItalic]) or (Style = [fsBold, fsItalic]) then
    begin
      tbItalic.Down    := True;
      tbItalic.Enabled := False;
    end
    else
    begin
      tbItalic.Down    := False;
      tbItalic.Enabled := True;
    end;
    FontComboBox.ItemIndex := 0;
    if Name = HyperText.FontNameA then FontComboBox.ItemIndex := 0;
    if Name = HyperText.FontNameR then FontComboBox.ItemIndex := 2;
    if Name = HyperText.FontNameV then FontComboBox.ItemIndex := 3;
    if Name = HyperText.FontNameG then FontComboBox.ItemIndex := 4;
    if Name = HyperText.FontNameM then FontComboBox.ItemIndex := 5;
   {$ifdef Linux}
    if Name = HyperText.FontNameS then
    begin
      tbItalic.Enabled := False;
      pmItalic.Enabled := False;
      FontComboBox.ItemIndex := 1;
    end
    else
    begin
      tbItalic.Enabled := True;
      pmItalic.Enabled := True;
    end;
    {$endif}
    //Self.DefaultFont.Name := Name;
    SizeComboBox.ItemIndex := 4;
    j := Trunc(Size / HyperText.MagScreen);
    if j <= 12 then
      SizeComboBox.ItemIndex := j - 7
    else
    begin
      if j = 14 then SizeComboBox.ItemIndex := 6;
      if j = 16 then SizeComboBox.ItemIndex := 7;
      if j = 20 then SizeComboBox.ItemIndex := 8;
      if j >=24 then SizeComboBox.ItemIndex := 9;
    end;
  end;
   if HyperText.Subscript = 1 then
    begin
      tbSub.Down    := True;
      tbSub.Enabled := False;
    end
    else
    begin
      tbSub.Down    := False;
      tbSub.Enabled := True;
    end;
    if HyperText.Subscript = -1 then
    begin
      tbSuper.Down    := True;
      tbSuper.Enabled := False;
    end
    else
    begin
      tbSuper.Down    := False;
      tbSuper.Enabled := True;
    end;
    if HyperText.Shadow then
    begin
      tbShadow.Down    := True;
      tbShadow.Enabled := False;
    end
    else
    begin
      tbShadow.Down    := False;
      tbShadow.Enabled := True;
    end;
		ColorLookupComboBox.SelectColor := DummyImage.Canvas.Font.Color;
  {
  Memo.SelText  := IniSelText;
  Memo.SelStart := IniSelStart;
  Memo.SelLength:= IniSelLength;
  }
  AllowInsertion := True;
end;

procedure TTitleEditor.ColorLookupComboBoxChange(Sender: TObject);
var
  i: Integer;
  {$ifdef Linux}
  S1, S2 : WideString;
  {$else}
  S1, S2 : String;
  {$endif}
begin
  i := Memo.SelStart;
  if AllowInsertion then
  begin
    PrepareInsertion;
    with ColorLookupComboBox do
    begin
      if SelectColor = clBlack   then S1 := 'K';
      if SelectColor = clAqua    then S1 := 'A';
      if SelectColor = clBlue    then S1 := 'B';
      if SelectColor = clDkGray  then S1 := 'D';
      if SelectColor = clFuchsia then S1 := 'F';
      if SelectColor = clGreen   then S1 := 'G';
      if SelectColor = clLime    then S1 := 'L';
      if SelectColor = clMaroon  then S1 := 'M';
      if SelectColor = clNavy    then S1 := 'N';
      if SelectColor = clOlive   then S1 := 'O';
      if SelectColor = clPurple  then S1 := 'P';
      if SelectColor = clRed     then S1 := 'R';
      if SelectColor = clSilver  then S1 := 'S';
      if SelectColor = clTeal    then S1 := 'T';
      if SelectColor = clYellow  then S1 := 'Y';
    end;
    if Memo.SelText <> '' then
    begin
      with DummyImage.Canvas.Font do
      begin
        S2 := 'K';
        if Color = clAqua    then S2 := 'A';
        if Color = clBlue    then S2 := 'B';
        if Color = clDkGray  then S2 := 'D';
        if Color = clFuchsia then S2 := 'F';
        if Color = clGreen   then S2 := 'G';
        if Color = clLime    then S2 := 'L';
        if Color = clMaroon  then S2 := 'M';
        if Color = clNavy    then S2 := 'N';
        if Color = clOlive   then S2 := 'O';
        if Color = clPurple  then S2 := 'P';
        if Color = clRed     then S2 := 'R';
        if Color = clSilver  then S2 := 'S';
        if Color = clTeal    then S2 := 'T';
        if Color = clYellow  then S2 := 'Y';
      end;
      Memo.SelText := '[' + S1 + ']' + Memo.SelText + '[' + S2 + ']'
    end
    else
      Memo.SelText := '[' + S1 + ']';
  end;
  Memo.SelStart := i + Length(Memo.SelText);
  //Memo.SelLength := 0;
  Memo.SetFocus;
end;

procedure TTitleEditor.pmUndoClick(Sender: TObject);
begin
  Memo.Undo;
end;

procedure TTitleEditor.pmCutClick(Sender: TObject);
begin
  Memo.CutToClipboard;
end;

procedure TTitleEditor.pmCopyClick(Sender: TObject);
begin
  Memo.CopyToClipboard;
end;

procedure TTitleEditor.pmPasteClick(Sender: TObject);
begin
  Memo.PasteFromClipboard;
end;

procedure TTitleEditor.pmSelectAllClick(Sender: TObject);
begin
  Memo.SelectAll;
end;

end.
